/* 
 * libbmeps - Bitmap to EPS conversion library
 * Copyright (C) 2000 - Dirk Krause
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * In this package the copy of the GNU Library General Public License
 * is placed in file COPYING.
 */

#ifndef BMEPS_INC
#define BMEPS_INC 1

#if defined(WIN32)
#ifdef BMEPS_DLL
#ifdef MAKE_BMEPS_DLL
#define BMEPSDLL __declspec( dllexport)
#else
#define BMEPSDLL __declspec( dllimport)
#endif
#else
#define BMEPSDLL
#endif
#else /* ! WIN32 */
#define BMEPSDLL
#endif

#include <stdio.h>
#include <stdlib.h>
#ifdef FPTEX
#include <win32lib.h>
#define exit(x) _exit(x)
#endif

typedef struct {
  int psl; int col; int a85; int rl; int fl;
  int alpha; int trans; int altrig;
  int mix; int specbg; int bg_red; int bg_green; int bg_blue; int dsc_show;
} BmepsConfiguration;

#ifdef __cplusplus
extern "C" {
#endif

extern BMEPSDLL void bmeps_setup(
  int psl, int col, int a85, int rl, int fl,
  int alpha, int trans, int altrig,
  int mix, int specbg, int bg_red, int bg_green, int bg_blue, int dsc_show
);
extern BMEPSDLL void bmeps_cfg(char *options);
extern BMEPSDLL void bmeps_set_draft(int dr);
extern BMEPSDLL int  bmeps_get_pslevel(void);
extern BMEPSDLL int  bmeps_get_colored(void);
extern BMEPSDLL int  bmeps_get_enc_a85(void);
extern BMEPSDLL int  bmeps_get_enc_rl(void);
extern BMEPSDLL int  bmeps_get_enc_fl(void);
extern BMEPSDLL int  bmeps_get_draft(void);
extern BMEPSDLL int  bmeps_get_alpha(void);
extern BMEPSDLL int  bmeps_get_trans(void);
extern BMEPSDLL int  bmeps_get_altrig(void);
extern BMEPSDLL int  bmeps_get_dsc_comm(void);
extern BMEPSDLL void bmeps_configure(void);
extern BMEPSDLL void bmeps_version(FILE *out);
extern BMEPSDLL int bmeps_get_mix(void);
extern BMEPSDLL int bmeps_get_specbg(void);
extern BMEPSDLL int bmeps_get_bg_red(void);
extern BMEPSDLL int bmeps_get_bg_green(void);
extern BMEPSDLL int bmeps_get_bg_blue(void);


/*
	Support function used by handler functions
*/
extern BMEPSDLL void bmeps_bb(FILE *out, unsigned long w, unsigned long h);
extern BMEPSDLL void bmeps_header(FILE *out, char *name, unsigned long w, unsigned long h);
extern BMEPSDLL void bmeps_begin_image(FILE *out, unsigned long w, unsigned long h);
extern BMEPSDLL void bmeps_add_rgb(int red, int green, int blue);
extern BMEPSDLL void bmeps_add_gray(int gray);
extern BMEPSDLL void bmeps_add_trans(int alpha);
extern BMEPSDLL void bmeps_end_image(FILE *out);
extern BMEPSDLL void bmeps_draft(FILE *out, unsigned long w, unsigned long h);
extern BMEPSDLL void bmeps_footer(FILE *out);
extern BMEPSDLL void bmeps_set_trans(int flag);

extern BMEPSDLL void bmeps_get_cfg(BmepsConfiguration *bcp);
extern BMEPSDLL void bmeps_set_cfg(BmepsConfiguration *bcp);
extern BMEPSDLL void bmeps_copy_cfg(BmepsConfiguration *d, BmepsConfiguration *s);

extern BMEPSDLL char *bmeps_tmpname(void);
extern BMEPSDLL void bmeps_delete(char *filename);

/*
	Invocation points
*/
extern BMEPSDLL int  bmeps_run(FILE *out, FILE *in, char *name);
extern BMEPSDLL int  bmeps_run_bb(FILE *out, FILE *in, char *name);
extern BMEPSDLL int  bmeps_wh(FILE *in, char *name, unsigned long *w, unsigned long *h);
extern BMEPSDLL int  bmeps_can_handle(char *name);

#if 0
/*	
	Handler functions invoked by the above functions
*/
extern BMEPSDLL int  bmeps_png(FILE *out, FILE *in, char *name);
extern BMEPSDLL int  bmeps_png_bb(FILE *out, FILE *in, char *name);
extern BMEPSDLL int  bmeps_png_wh(FILE *in, unsigned long *w, unsigned long *h);
#else
#include "pngeps.h"
#endif

#ifdef __cplusplus
}
#endif
/* #ifdef __cplusplus */

#define MAXPSLEVEL 3

#endif
/* BMEPS_INC */

