/*
 * Copyright (c) 2000, Red Hat, Inc.
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     A copy of the GNU General Public License can be found at
 *     http://www.gnu.org/
 *
 * Written by DJ Delorie <dj@cygnus.com>
 *
 */

/* The purpose of this file is to manage internet downloads using the
   Internet Explorer version 5 DLLs.  To use this method, the user
   must already have installed and configured LibWWW.  This module is
   called from netio.cc, which is called from geturl.cc */

static char *cvsid = "\n%%% $Id: nio-libwww.cc,v 1.1.2.1 2000/12/11 22:01:41 andyp Exp $\n";

#include <stdlib.h>
#include "win32.h"

#include "resource.h"
#include "state.h"
// #include "dialog.h"
// #include "msg.h"
#include "netio.h"
#include "nio-libwww.h"

static int retcode = 0;
static int progress = 0;

/*
**  We get called here from the event loop when we are done
**  loading. Here we terminate the program as we have nothing
**  better to do.
*/
static int terminate_handler (HTRequest * request, HTResponse * response,
			      void * param, int status)
{
    /* Delete our request again */
    HTRequest_delete(request);

    /* Delete our profile */
    HTProfile_delete();

    retcode = status;
}

BOOL read_progress_handler(HTRequest * request, HTAlertOpcode op,
			   int msgnum, const char * dfault, void * input,
			   HTAlertPar * reply)
{
  return TRUE;
}


NetIO_LibWWW::NetIO_LibWWW (char *_url)
  : NetIO (_url)
{

    HTRequest *         request = NULL;

    /* Initiate W3C Reference Library with a client profile */
    HTProfile_newNoCacheClient(APP_NAME, APP_VERSION);

    /* And the traces... */
#if 0
    HTSetTraceMessageMask("sop");

    /* Need our own trace and print functions */
    HTPrint_setCallback(printer);
    HTTrace_setCallback(tracer);
#endif

    /* Add our own filter to terminate the application */
    HTNet_addAfter(terminate_handler, NULL, NULL, HT_ALL, HT_FILTER_LAST);

#if 1
    /* Add our own alerter to capture progress download */
    HTAlertCall_add (HTAlert_global(), read_progress_handler,
		     HT_PROG_READ);
#endif

    /* Set the timeout for how long we are going to wait for a response */
    HTHost_setEventTimeout(10000);

    request = HTRequest_new();
    
    // What's this ?
    // HTRequest_setOutputStream(request, HTContentCounter(HTBlackHole(), request, 0x2000));
    
    /* Start the load */
    if (HTLoadToFile(getme, request, outputfile) != YES) {
      HTPrint("Can't open output file\n");
      HTProfile_delete();
      return 0;
    }

    /* Go into the event loop... */
    HTEventList_loop(request);

}

void
NetIO_LibWWW::flush_io ()
{
}

NetIO_LibWWW::~NetIO_LibWWW ()
{
}

int
NetIO_LibWWW::ok ()
{
  return (request == NULL) ? 0 : 1;
}

int
NetIO_LibWWW::read (char *buf, int nbytes)
{
}
