#
# Build tool for Web2C & supp
# Author: F. Popineau
# Date  : 04/11/98
# Time-stamp: <04/11/19 23:31:19 popineau>
#
# Usage: 
#        build <option>
#              [--static]         builds/cleans up statically
#              [--debug]          builds/cleans up in debug mode
#              [--depend]         updates dependencies
#              [--clean]          cleans up 
#              [--distclean]      removes everything that can be rebuild (needs to configure again)
#              [--dir <dir>]      only in <dir> (relative to top !)
#              [--install]        installs what has been built (incompaible with clean up targets)
#              [--from-scratch]   full build, removes everything from the destination tree.
#              [--log <file>]     logs everything onto <file>
#
#        Hope these options are clear enough. This is a wrapper to nmake calls.
#

BEGIN {

  my $path = 'c:\\program files\\texlive\\bin\\win32;c:\\program files\\imagemagick;' . $ENV{'PATH'};
  $ENV{'PATH'} = $path;
}

require "newgetopt.pl";
use lib qw (. ./msvc);
use FileUtils qw(canon_dir cleandir make_link newpath 
		  normalize substitute_var_val dirname
		  rec_rmdir sync_dir walk_dir start_redirection
		  stop_redirection sync_dir);
use Tpm;
use Cwd;
use Time::gmtime;

my $fpTeXVersion = "0.7";
my $FtpDir = "c:/InetPub/ftp/fptex/$fpTeXVersion";

my $Gm = gmtime();
my $Date = sprintf("%02d-%02d-%02d", $Gm->year() + 1900, $Gm->mon(), $Gm->wday());
# root directory
my $topdir = &dirname(cwd());
my $win32_dir = "$topdir/msvc";
my $log_dir = "$topdir/tmp/logs";
# my $msvc_dir = "$topdir/texk/msvc";
my $zipsrc_name = "$FtpDir/sources/fptex-$fpTeXVersion-$Date.zip";
my $logfile = "";
my $prefix;

local $opt_test;
local $opt_verbose;

# the directories for which we build standalone packages
# (aka: no DLL)
@src_standalone = ( 
		   "libs/libgnuw32",
		   "libs/libgsw32",
		   "libs/geturl", 
		   "libs/zlib",
		   "graphics/libpng",
		   "graphics/jpeg",
		   "libs/xpdf",
		   "gnu/bzip2",
		   "gnu/gzip",
		   "graphics/jpeg2ps",
		   "texk/kpathsea",
		   "texk/xemtex",
		   "texk/texmfmp",
		   "texk/web2c",
		   "texk/tie",
		   "texk/web",
		   "texk/mp",
		   "texk/mpware",
		   "texk/pdftexdir",
#		   "texk/pdftex",
		   "texk/pdfetex",
		   "texk/pdfxtex",
		   "texk/omega",
		   "texk/aleph",
#		   "texk/omegafonts",
#		   "texk/omegaware",
#		   "texk/otps",
		   "texk/windvi",
		   "texk/dvipsk",
#		   "texk/odvipsk",
#		   "texk/owindvi",
		   "texk/gsftopk",
		   "texk/tex4htk",
		   "texk/dvipdfm",
		   "texk/dvipdfmx",
#		   "texk/dvipos",
		   "texk/lcdf-typetools",
		   "texk/t1utils",
		   "texk/afm2pl"
		   );

# directories that should not end up in the zip archive
# of sources
@zipsrc_exclude_dirs = (
			"/(dynamic|dynamic-debug|static|static-debug|Release|Debug)\$",
			"/win32/(diff|setupw32|factory|ttfinst|worked-on)\$",
			"/win32/(deutsch|english|fptex.old|french|labels|slovak|tds)\$",
			"/utils/rxp-1.2\$",
			"/texk/(ltx2rtf-old|windvi-0.68|texdoctk-0.3.0|texdoctk-0.4.0)\$"
		       );

@zipsrc_exclude_patterns = (
 "\.(obj|dll|exe|opt)\$",
 "~\$"
);


my %modules = ('tex' => ['libs', 'graphics', 'gnu', 'texk'],
	       'texlive' => ['libs', 'graphics', 'perl', 'gstools', 'texk'],
	       'gs' => ['libs', 'graphics', 'gstools'],
	       'perl' => ['libs', 'perl'],
	       'xemacs' => ['libs', 'graphics', 'gnu', 'xemacs'],
	       'standalone' => \@src_standalone,
	       'xemtex' => ['libs', 'graphics', 'gnu', 'perl', 'gstools', 'xemacs', 'texk', 'setup']
	      );

my %targets = ('build' => ['all'],
	       'install' => ['formats', 'install', 'install-man', 'install-doc'],
	       'man' => ['install-man'],
	       'doc' => ['install-doc'],
	       'clean' => ['clean'],
	       'distclean' => ['distclean'],
	       'depend' => ['depend'],
	       'standalone' => ['all', 'install-exec']
	      );

&main;

1;

sub help
{
    print "Usage: build [options]\n";
    print "\n";
    print "    --dist=(texlive|xemtex)	choose the target directory;\n";
    print "    --module=tex		only libs + gnu + graphics + texk;\n";
    print "            =gs		only libs + graphics + gstools;\n";
    print "            =perl		only libs + perl;\n";
    print "            =xemacs		only libs + gnu + graphics +xemacs;\n";
    print "            =all		all modules;\n";
    print "    --target=build		build the module;\n";
    print "            =install		build and install the module;\n";
    print "            =man		install man pages;\n";
    print "            =doc		install doc files;\n";
    print "            =clean		clean up the module;\n";
    print "            =distclean	really clean up the module;\n";
    print "            =depend		rebuild the `depend.mak' files;\n";
    print "    --static    		build static versions;\n";
    print "    --debug			build debug versions;\n";
    print "    --from_scratch		install module from scratch, available with `all' module;\n";
    print "    --sync           	sync up with master tree (with tex or all module);\n";
    print "    --standalone     	build standalone packages (ie: static, nodebug);\n";
    print "    --zipsrc(=clean) 	build a source archive, (possibly cleaning the tree before);\n";
    print "\n";
    print "    Support commands:\n";
    print "    --log	           	generate a log file of execution;\n";
    print "    --verbose        	some more messages\n";
    print "    --help           	display this help text and exits\n";
}

sub mask_env {
  my ($value) = @_;
  $ENV{"SystemDrive"} = "${value}";
  $ENV{"TEXMFCNF"} = "${value}";
  $ENV{"TEXMFMAIN"} = "${value}";
#  $ENV{"TEXMF"} = "${value}";
  $ENV{"LATEX2HTML"} = "${value}";
  $ENV{"GROFF_TMAC_PATH"}="${value}";
  $ENV{"GROFF_FONT_PATH"}="${value}";
}

sub ziplist {
  my ($dir, @l) = @_;
  $dir =~ s@\\@/@g;
  map {
    if (! -d "$dir/$_") {
      for $pattern (@zipsrc_exclude_patterns) {
	if ("$dir/$_" =~ m@$pattern@i) {
	  goto suivant;
	}
      }
      print STDERR "$dir/$_\n" if ($opt_verbose);
      print ZIP "$dir/$_\n";
    }
  suivant:
  } @l;
}

sub zipprune {
  my ($dir) = @_;
  $dir =~ s@\\@/@g;
  map {
    if ($dir =~ m@$_@) {
      print STDERR "Pruning $dir...\n" if ($opt_verbose);
      return 1;
    }
  } @zipsrc_exclude_dirs;
  return 0;
}

sub do_zipsrc {
  unlink $zipsrc_name if (-f $zipsrc_name);
  open ZIP, "| zip -9\@or " . $zipsrc_name . " > nul";
  &walk_dir($topdir, \&ziplist, \&zipprune);
  close ZIP;
}

sub do_standalone {
  my ($prog) = @_;
  my($cmd, $cwd, $d);

  # make the static binaries needed
  print STDERR "Installing in $prefix\n";

  &mask_env('foo');

  &start_redirection($logfile);

  # We need to build texinfo first.
  $d = "texk/texinfo";
  $cwd = &getcwd;
  print "Changing directory to $topdir/${d}\n" if ($opt_verbose);
  chdir("$topdir/${d}");
  $cmd = $prog . "all";
  print "Running $cmd\n" if ($opt_verbose);
  system($cmd);
  chdir($cwd);
  
  for $d (@src_standalone) {
    $cwd = &getcwd;
    print "Changing directory to $topdir/${d}\n" if $opt_verbose;
    chdir("$topdir/${d}");
    $cmd = $prog . "all";
    print "Running $cmd\n" if $opt_verbose;
    system($cmd);
    $cmd = $prog . " STANDALONE=1 install-dirs";
    print "Running $cmd\n" if $opt_verbose;
    system($cmd);
    $cmd = $prog . " STANDALONE=1 install-exec";
    print "Running $cmd\n" if $opt_verbose;
    system($cmd);
    chdir($cwd)
  }

  &stop_redirection($logfile);
}

sub system_bypass {
  my ($cmd) = @_;
  if (${opt_test} || ${opt_verbose}) {
    print "Doing ${cmd}\n";
  }
  if (! ${opt_test}) {
    system($cmd);
  }
}

sub main {

  unless (&NGetOpt (
		    "module=s",
		    "target=s",
		    "static",
		    "debug",
		    "from_scratch",
		    "sync",
		    "standalone",
		    "sync",
		    "zipsrc:s",
		    "test",
		    "log",
		    "verbose",
		    "help"))
    {
	print STDERR "Try `configure --help'";
	exit 1;
    }

  if ($opt_help) {
    &help;
    exit 0;
  }

  # fix PATH for the tools we use
  print $ENV{"PATH"} . "\n";

  # build the command line
  my $prog = "nmake -f win32.mak ";
  $prog .= "verbose=on " if ($opt_verbose);
  
  if ($opt_module eq 'texlive') {
    $prog .= "TEXLIVE=1 ";
    $prefix = "c:/Program Files/TeXLive";
  }
  elsif ($opt_module eq 'xemtex') {
    $prog .= "XEMTEX=1 ";
    $prefix = "c:/Program Files/XEmTeX";
  }
  elsif ($opt_module eq 'xemacs') {
    $prog .= "XEMACS=1 ";
    $prefix = "c:/Program Files/XEmacs";
  }
  else {
    die "You need to chose a `dist' directory!\n";
  }

  my @subdirs = @{$modules{${opt_module}}};

  my @target_list = @{$targets{${opt_target}}};
  die "You need to chose a target!" if (! @target_list);

  if ($opt_log) {
    $logfile = "$log_dir\\${opt_module}-${opt_target}.log";
  }

  if ($opt_debug eq '') {
    $prog .= "NODEBUG=1 ";
  }

  if ($opt_static) {
    $prog .= "STATIC=1 ";
  }

  if (&FileUtils::member('depend', @target_list)) {
    $prog .= "MAINT=1 ";
  }

  if ($opt_sync) {
    $prog .= "SYNC=1 ";
  }

  if (${opt_module} eq 'standalone') {
    $prog .= "STATIC=1 STANDALONE=1 ";
  }

  print STDERR "Installing in $prefix\n";

  &mask_env('');

  # start redirection if asked
  &start_redirection($logfile);

  if ($opt_from_scratch) {

    my ($dir) = &canon_dir($prefix, '\\');
    print "Prefix directory is $dir .\n" if $opt_verbose;

    if (${opt_target} eq 'install') {
      &cleandir("$dir\\bin\\win32");
      &cleandir("$dir\\xemacs-21.5\\i586-pc-win32");
      &cleandir("$dir\\bin\\win32-static");
    }
    else {
      print "Not cleaning up bin directories, because module is not `all' or target is not `install'!\n";
    }
  }

  if (${opt_sync}) {
    if ((${opt_target} eq 'install')) {
      # FIXME: do it using modules
      my $cmd = "perl ./sync.pl --texmf=${opt_module} --direction=from";
      &system_bypass($cmd);
      &system_bypass("\"${prefix}/bin/win32/mktexlsr.exe\"");
    }
    else {
      print STDERR "Not doing sync because target is not `install' or module is not `all' or `tex'.\n";
    }
  }

  if (&FileUtils::member(${opt_target}, ('all', 'distclean', 'clean'))) {
    unlink("./perlfiles.lst");
  }

#   if (${opt_target} eq 'install' && (&FileUtils::member(${opt_module}, ('all', 'tex')))) {
#       &system_bypass("\"${prefix}/bin/win32/fmtutil.exe\" --all --dolinks --force");
#   }

  &mask_env('foo');

  # save directory
  my $olddir = &getcwd;

  for my $target (@target_list) {
    if ($target eq 'formats') {

      &mask_env('');

      if (${opt_target} eq 'install' && (&FileUtils::member(${opt_module}, ('all', 'tex')))) {
	&system_bypass("\"${prefix}/bin/win32/fmtutil.exe\" --all --dolinks --force");
      }

      &mask_env('foo');

    }
    else {
      for my $dir (@subdirs) {
	chdir("$topdir/$dir");
	# Call the program
	#`$prog`;
	print STDOUT "Executing `$prog $target' in " . &getcwd . "\n";
	&system_bypass($prog . $target);
      }
    }
  }
  chdir($olddir);

  &mask_env('');

  if (${opt_sync}) {

#    if (${opt_module} eq 'texlive') {

#       if (${opt_target} eq 'install' 
# 	  && (&FileUtils::member(${opt_module}, ('all', 'tex')))) {
# 	# FIXME: do it using modules
# 	my $cmd = "perl ./sync.pl --texmf=${opt_module} --direction=to";
#       &system_bypass($cmd);
#       }
#       else {
# 	print STDERR "Not doing sync because target is not `install' or module is not `all' or `tex'.\n";
#       }
      
      if (${opt_target} eq 'install') {
	if ($opt_module eq 'standalone' || $opt_static) {
	  $cmd = "perl ./sync.pl \"${prefix}/bin/win32-static\" ${Tpm::MasterDir}/bin/win32-static";
	  &system_bypass($cmd);
	}
	else {
	  $cmd = "perl ./sync.pl \"${prefix}/bin/win32\" ${Tpm::MasterDir}/bin/win32";
	  &system_bypass($cmd);
	}
      }
#    }
#    else {
#      print STDERR "Not doing sync because dist is not `texlive'.\n";
#    }
  }

  # stop redirection
  stop_redirection($logfile);

  # restore directory
  chdir $topdir;

  if ($opt_distclean && $opt_zipsrc) {
    &do_zipsrc;
  }

  print "This is the end!\n";
}
