#
# Diff tool for Web2C & supp
# Author: F. Popineau
# Date  : 01/09/04
# Time-stamp: <04/11/08 13:35:56 popineau>
#
# Usage: 
#

BEGIN {
push @INC, 'c:/source/XEmTeX/msvc';
}

require "newgetopt.pl";
use lib qw (. ./msvc);
use FileUtils qw(canon_dir cleandir make_link newpath 
		  normalize substitute_var_val dirname
		  rec_rmdir sync_dir walk_dir start_redirection
		  stop_redirection sync_dir);
use Tpm;
use Cwd;
use Time::gmtime;

my $opt_rootdir = "c:/source/XEmTeX";
my $opt_diffdir = "${opt_rootdir}/tmp/diffs";
my $opt_win32subdir = "${opt_rootdir}/msvc";
my $opt_depotdir = "c:/source/TeXLive";
my $opt_source = "source.development";

my %modules = (
	       'gnu/bzip2' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/bzip2",
	       'gnu/gzip' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/gzip",
	       'graphics/gd' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/gd",
	       'graphics/jpeg' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/jpeg",
	       'graphics/jpeg2ps' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/jpeg2ps",
	       'graphics/libpng' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/libpng",
	       'libs/expat' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/expat",
	       'libs/freetype' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/freetype",
	       'libs/freetype2' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/freetype2",
	       'libs/geturl' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/geturl",
	       'libs/gifreader' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/gifreader",
	       'libs/libgnuw32' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/libgnuw32",
	       'libs/libgsw32' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/libgsw32",
	       'libs/libttf' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/libttf",
	       'libs/md5' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/md5",
	       'libs/regex' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/regex",
	       'libs/t1lib' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/t1lib",
	       'libs/type1' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/type1",
	       'libs/unzip' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/unzip",
	       'libs/xpdf' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/xpdf",
	       'libs/zlib' => "${opt_depotdir}/Build/${opt_source}/TeX/libs/zlib",
	       'texk/afm2pl' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/afm2pl",
	       'texk/aleph' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/alephdir",
	       'texk/bibtex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/bibtex8' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/bibtex8",
	       'texk/chktex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/chktex",
	       'texk/cjkutils' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/cjkutils",
	       'texk/ctie' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/ctiedir",
	       'texk/cweb' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/cwebdir",
	       'texk/detex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/detex",
	       'texk/devnag' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/devnag",
	       'texk/doc' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/doc",
	       'texk/dtl' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dtl",
	       'texk/dvi2tty' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dvi2tty",
	       'texk/dvicopy' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/dvidvi' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dvidvi",
	       'texk/dviljk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dviljk",
	       'texk/dvipdfm' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dvipdfm",
	       'texk/dvipdfmx' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dvipdfmx",
	       'texk/dvipng' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dvipng",
	       'texk/dvipos' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dvipos",
	       'texk/dvipsk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/dvipsk",
	       'texk/etex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/etexdir",
	       'texk/fontware' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/gsftopk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/gsftopk",
	       'texk/kpathsea' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/kpathsea",
	       'texk/lacheck' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/lacheck",
	       'texk/latexmk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/latexmk",
	       'texk/lcdf-typetools' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/lcdf-typetools",
	       'texk/makeindexk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/makeindexk",
	       'texk/mf' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/mfware' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/mp' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/mpware' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/mpware",
	       'texk/musixflx' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/musixflx",
	       'texk/odvipsk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/odvipsk",
	       'texk/omega' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/omegadir",
	       'texk/omegafonts' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/omegafonts",
	       'texk/omegaware' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/omegaware",
	       'texk/otps' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/otps",
	       'texk/owindvi' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/owindvi",
	       'texk/patgen' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/pdfetex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/pdfetexdir",
	       'texk/pdftex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/pdftexdir",
	       'texk/pdftexdir' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/pdftexdir",
	       'texk/pdfxtex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/pdfxtexdir",
	       'texk/ps2pkm' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/ps2pkm",
	       'texk/psutils' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/psutils",
	       'texk/seetexk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/seetexk",
	       'texk/t1utils' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/t1utils",
	       'texk/tests' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/tests",
	       'texk/tex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/tex4htk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/tex4htk",
	       'texk/texdoctk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/texdoctk",
	       'texk/texi2html' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/texi2html",
	       'texk/texinfo' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/texinfo",
	       'texk/texmfmp' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/lib",
	       'texk/texware' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/tie' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/tiedir",
	       'texk/triptrap' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/triptrap",
	       'texk/ttf2pfb' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/ttf2pfb",
	       'texk/ttf2pk' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/ttf2pk",
	       'texk/ttf2pt1' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/ttf2pt1",
	       'texk/ttfdump' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/ttfdump",
	       'texk/tth' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/tth",
	       'texk/vlna' => "${opt_depotdir}/Build/${opt_source}/TeX/utils/vlna",
	       'texk/web' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c",
	       'texk/web2c' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/web2c/web2c",
	       'texk/windvi' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/windvi",
	       'texk/xemtex' => "${opt_depotdir}/Build/${opt_source}/TeX/texk/xemtex",
	       'xemacs/xemacs-21.5' => "c:/mirror/xemacs/xemacs",
	      );
&main;

exit(0);

sub help {
  print "Usage: $0 [options]\n";
}

sub main {

  die "Non-existent directory ${opt_diffdir}\n" if (! -d ${opt_diffdir});
    
  for my $subdir (keys %modules) {

    if (! -d $modules{$subdir}) { 
      print "Non-existent directory: $modules{$subdir}\nSkipping...\n"; next; 
    }
    my $filename = $subdir;
    $filename =~ s/\//_/g;
    my $cmd = "diff -ru -X ${opt_win32subdir}/diff.exclude ${opt_rootdir}/${subdir} " . $modules{$subdir} . " > ${opt_diffdir}/${filename}.diff";
    print "${cmd}\n" if ($opt_verbose);
    system($cmd);
  }
}
