require "newgetopt.pl";
use lib ("c:/source/xemtex/msvc");
use Cwd;
use FileUtils qw(dirname basename normalize walk_dir cleandir sort_uniq
		 copy remove_list push_uniq newer sync_dir canon_dir
		 is_absolute build_path);
use File::Copy qw(copy);
use Tpm;

my @ignore = ( 
	   "libs/gifreader",
	   "libs/geturl",
	   "libs/geturl-new",
	   "libs/freetype2",
	   "libs/expat",
	   "libs/libttf",
	   "libs/T1",
	   "libs/unzip",
	   "texk/kpathsea-fp",
	   "texk/kpsexx",
	   "texk/tth",
	   "texk/windvi-0.68",
	   "utils/l2h-orig",
	   "utils/l2h-orig-orig",
	   "utils/latex2html",
	   "utils/noweb",
	   "utils/src2tex",
	   "utils/texinfo/info",
	   "texk.stable",
	   "texk/kpse-shared*",
	   "texk/cwebk",
	   "texk/msvc/factory",
	   "texk/kpse.*",
	   "texk/eomegadir/.*",
	   "texk/eomegadir-1.15/.*",
	   "texk/eomegadir-1.23/.*",
	   "CVS"
	  );

# $ignore_regexp = "(" . join ( '|', map { s/^/.\\/ ; s/\//\\/g; $_ } @ignore) . ")";
my $ignore_regexp = "(" . join ( '|', @ignore) . ")";

&main;

1;

sub restrict_win {
  my ($dir, $dst, @files) = @_;
  # under the texk/msvc dir, we don't want to copy all the files
  $dir =~ s/^\.//;
  $dir = &getcwd . $dir;
  $dir =~ s@\\@/@g;
  if ($dir =~ m@/msvc$@) {
    @files = grep { /(\.h|\.mak|\.sed|\.pl|\.pm|\.bat|\.inc|\.version)$/ } @files;
  } 
  # only .mak and .rc files under most directories
  elsif ($dir eq '.' || ($dir =~ m@/(texk|libs|utils)@ && $dir !~ m@$ignore_regexp@xo)) {
#    map { if ($dir =~ m@[/\\]$_@) { @files = (); return @files; } } @ignore;
    @files = grep { (-d "$dir/$_") || ($_ =~ m/(\.mak|\.rc)$/) } @files;
  }
  # don't bother with other directories
  else {
    @files = ();
  }
  return @files;
}

sub restrict_unix {
  my ($dir, $dst, @files) = @_;
  $dir =~ s/^\.//;
  $dir = &getcwd . $dir;
  $dir =~ s@\\@/@g;
  # Unix autoconf files
  if ($dir eq '.' || $dir =~ m@[/\\](texk|libs|utils)@ && $dir !~ m@$ignore_regexp@xo) {
    @files = grep { /(Makefile\.in|config|configure|configure\.in|config\.guess|config\.sub|c-auto\.in|\.mk)$/ || (-d "$dir/$_") } @files;
  }
  # don't bother with other directories
  else {
    @files = ();
  }
  return @files;
}

sub restrict_mswin32 {
  my ($dir, $dst, @files) = @_;

  @files = grep /(README|\.exclude|\*.pm|\.pl|\.bat|\.txt)$/, @files;

}

sub restrict_standard {
  my ($dir, $dst, @files) = @_;
  $dir =~ s/^\.//;
  $dir = &getcwd . $dir;
  $dir =~ s@\\@/@g;

  if ($dir =~ m@/CVS$@) {
    @files = ();
  }
  else {
    @files = grep { $_ !~ /^(\.nosearch|\.cvsignore)$/ } @files ;
  }
  return @files;
}

sub collect_files {
  local ($d, @lf) = @_;
  my ($f);
  foreach $f (@lf) {
    my $path = "$d/$f";
    if (-f "$path") {
      $path =~ s@^${dstdir}/@@;
      $path =~ s@\\@/@g;
      if ($path !~ m@^${Tpm::IgnoredFiles}$@
	 && !($path =~ m@^bin/@ && $path !~ m@^bin/win32/@)) { 
	   push @dstfiles, $path; 
	 }
    }
  }
}

sub main {
  
  unless (&NGetOpt ("dry", "mirror", "sources", "texmf=s", "restrict", "direction=s", "force")) {
    print STDERR "Try `$0 --help'";
    exit 1;
  }

  $opt_verbose = 1;

  if ($opt_sources) {
    local ($f1, $f2);
    $f1 = "c:/source/XEmTeX/libs";
    $f2 = "c:/source/texlive/Build/source/TeX/libs";
    &sync_dir($f1, $f2, \&restrict_win, "", $opt_dry != 0, $opt_mirror != 0, 1);
    &sync_dir($f2, $f1, \&restrict_unix, "", $opt_dry != 0, $opt_mirror != 0, 1);
    $f2 = "c:/source/texlive/Build/source.development/TeX/libs";
    &sync_dir($f1, $f2, \&restrict_win, "", $opt_dry != 0, $opt_mirror != 0, 1);
    &sync_dir($f2, $f1, \&restrict_unix, "", $opt_dry != 0, $opt_mirror != 0, 1);

    $f1 = "c:/source/XEmTeX/utils";
    $f2 = "c:/source/texlive/Build/source/TeX/utils";
    &sync_dir($f1, $f2, \&restrict_win, "", $opt_dry != 0, $opt_mirror != 0, 1);
    &sync_dir($f2, $f1, \&restrict_unix, "", $opt_dry != 0, $opt_mirror != 0, 1);
    $f2 = "c:/source/texlive/Build/source.development/TeX/utils";
    &sync_dir($f1, $f2, \&restrict_win, "", $opt_dry != 0, $opt_mirror != 0, 1);
    &sync_dir($f2, $f1, \&restrict_unix, "", $opt_dry != 0, $opt_mirror != 0, 1);

    $f1 = "c:/source/XEmTeX/texk";
    $f2 = "c:/source/texlive/Build/source/TeX/texk";
    &sync_dir($f1, $f2, \&restrict_win, "", $opt_dry != 0, $opt_mirror != 0, 1);
    &sync_dir($f2, $f1, \&restrict_unix, "", $opt_dry != 0, $opt_mirror != 0, 1);

    $f1 = "c:/source/XEmTeX/texk.development";
    $f2 = "c:/source/texlive/Build/source.development/TeX/texk";
    &sync_dir($f1, $f2, \&restrict_win, "", $opt_dry != 0, $opt_mirror != 0, 1);
    &sync_dir($f2, $f1, \&restrict_unix, "", $opt_dry != 0, $opt_mirror != 0, 1);

    $f1 = "c:/source/XEmTeX/msvc";
    $f2 = "c:/source/texlive/Build/source/TeX/mswin32";
    &sync_dir($f1, $f2, \&restrict_mswin32, "", $opt_dry != 0, $opt_mirror != 0, 1);

    $f1 = "c:/source/XEmTeX/msvc";
    $f2 = "c:/source/texlive/Build/source.development/TeX/mswin32";
    &sync_dir($f1, $f2, \&restrict_mswin32, "", $opt_dry != 0, $opt_mirror != 0, 1);

  }
  elsif ($opt_texmf =~ /^(texlive|xemtex)$/) {
    my $f1, $f2;
    local $srcdir, $dstdir;
    if ($opt_texmf eq 'texlive') {
      ($f1, $f2) = ("${Tpm::MasterDir}", "C:/Program Files/TeXLive");
    }
    elsif ($opt_texmf eq 'xemtex') {
      ($f1, $f2) = ("${Tpm::MasterDir}", "C:/Program Files/XeMTeX");
    }
    else {
      die "Bad option --texmf=$opt_texmf...\n";
    }
    if ($opt_direction eq 'from') {
      ($srcdir, $dstdir) = ($f1, $f2);
    }
    elsif ($opt_direction eq 'to') {
      ($srcdir, $dstdir) = ($f2, $f1);
      ${Tpm::MasterDir} = $srcdir;
    }
    else {
      die "No direction given for syncing...\n";
    }
    my $tpm;
    if ($opt_texmf eq 'texlive') {
      $tpm = Tpm->new("${srcdir}/texmf/tpm/scheme-full.tpm");
    }
    # Why not get the list of files and apply regexpify() ?
    elsif ($opt_texmf eq 'xemtex') {
      $tpm = Tpm->new("${srcdir}/texmf/tpm/scheme-xemtex.tpm");
    }
    ${Tpm::CurrentArch} = 'win32';
#    local @srcfiles = grep { /^texmf(-dist|-doc)?\// } $tpm->getRequiredFileList("all");
    local @srcfiles = &FileUtils::sort_uniq($tpm->getRequiredFileList("all"));
    local @dstfiles = ();
#    print "******** srcfiles ********\n";
#    map { print "$_\n"; } @srcfiles; 
#    print "******** srcfiles ********\n";
    &FileUtils::walk_dir("$dstdir/bin", \&collect_files);
    &FileUtils::walk_dir("$dstdir/texmf", \&collect_files);
    &FileUtils::walk_dir("$dstdir/texmf-dist", \&collect_files);
    &FileUtils::walk_dir("$dstdir/texmf-doc", \&collect_files);
    local @l1 = (); 
    local @l2 = ();
    &FileUtils::diff_list(\@srcfiles, \@dstfiles, \@l1, \@l2);
#    print "******** srcfiles ********\n";
#    map { print "$_\n"; } @srcfiles; 
#    print "******** srcfiles ********\n";
#    print "******** dstfiles ********\n";
#    map { print "$_\n"; } @dstfiles; 
    print "******** l1 ********\n";
    map { print "$_\n"; } @l1; 
    print "******** l2 ********\n";
    map { print "$_\n"; } @l2; print "\n";
    my @toberemoved = &FileUtils::regexpify(1, "$dstdir", @l2);
    print "******** toberemoved ********\n";
    map { print "$_\n"; } @toberemoved; 

    map {
      my $path = "$dstdir/$_";
      if (-f $path) {
	print "Removing file $path\n";
	unlink $path if ($opt_dry == 0);
      }
      elsif ($path =~ m@^(.*)/\*$@) {
	$path = $1;
	print "Removing directory $path\n";
	&FileUtils::rec_rmdir($path) if ($opt_dry == 0);
      }
      else {
	print "!!!Don't know how to remove $path\n";
      }
    } @toberemoved;

    map {
      my $path = $_;
#      print "Checking $srcdir/$path versus $dstdir/$path\n";
      if (-f "$srcdir/$path") {
	($f1, $f2) = ("$srcdir/$path", "$dstdir/$path");
	if (! -f $f2 || $opt_force || &FileUtils::newer($f1, $f2) > 0) {
	  if (! -f $f2) {
	    print "Copying non-existing $f2\n";
	  }
	  else {
	    print "Copying newer file $f1 than $f2\n";
	  }
	  if ($opt_dry == 0) {
	    my $parentdir = &FileUtils::dirname($f2);
	    if (! -d $parentdir) {
	      print "Creating directory $parentdir\n";
	      &FileUtils::rec_mkdir($parentdir);
	    }
	    &FileUtils::copy($f1, $f2);
	  }
	}
      }
      else {
	print "!!!Source file does not exist: $path\n";
      }
	
    } @srcfiles;
  }
  else {
    local ($f1, $f2) = @ARGV;

    die "$f1 is not a directory!\n" if (! -d $f1);
    die "$f2 is not a directory!\n" if (! -d $f2);

    $f1 = &canon_dir( &build_path(&getcwd, "$f1")) if (! &is_absolute($f1));
    $f2 = &canon_dir( &build_path(&getcwd, "$f2")) if (! &is_absolute($f2));
    
    if ($opt_restrict) {
      &sync_dir($f1, $f2, \&restrict_standard, "", $opt_dry != 0, $opt_mirror != 0, 0);
    }
    else {
      &sync_dir($f1, $f2, "", "", $opt_dry != 0, $opt_mirror != 0, 0);
    }
  }
}

