################################################################################
#
# Makefile  : texi, build info, pdf and html files out of texinfo files
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/09/02 16:35:32 popineau>
#
################################################################################

# paths to doc directories

!ifndef docdir
docdir = $(texmf)\doc
!endif

!ifndef infodir
infodir = $(docdir)\info
!endif

!ifndef htmldocdir
htmldocdir = $(docdir)\$(docsubdir)
!endif

!ifndef pdfdocdir
pdfdocdir = $(docdir)\$(docsubdir)
!endif

#
# Only if infofiles has been defined
#

!if ("$(infofiles)" != "" || "$(pdfdocfiles)" != "" || "$(htmldocfiles)" != "")

makeinfo = $(texk_srcdir)\texinfo\makeinfo\$(objdir)\makeinfo.exe
makeinfo_flags = --force --paragraph-indent=2 --no-split -I$(srcdir)

install_info = $(texk_srcdir)\texinfo\util\$(objdir)\install-info.exe
install_info_flags =

texindex = $(texk_srcdir)\texinfo\util\$(objdir)\texindex.exe
texindex_flags =

# texi2html = perl $(texk_srcdir)\texi2html-1.64\texi2html.pl
# texi2html_flags = -expandinfo -number -menu -split_chapter -subdir=html
texi2html = $(makeinfo)
texi2html_flags = --html --force --no-split -I$(srcdir)
# If you prefer one big .html file instead of several, remove
# -split-node or replace it by -split_chapter.

# For making normal text files out of Texinfo source.
one_info = --no-headers --no-split --no-validate

dvips = dvips
dvipsflags =
texflags=-interaction=batchmode

.SUFFIXES: .info .dvi .html .texi .txi .pdf .tex .texinfo

.texinfo.texi:
	-@$(copy) $< $@

.texi.info:
	-$(silent)$(makeinfo) $(makeinfo_flags) $< -o $@

.texi.dvi:
#	$(TEXI2DVI) $(TEXI2DVI_FLAGS) $<
	-@echo $(verbose) & echo "Compiling $@ out of $<" & ( \
	etex $(texflags) $< & \
	etex $(texflags) $< & \
	$(texindex) $(<:.texi=.??) & \
	etex $(texflags) $< ) $(redir_stdout)
	-@$(del) $(<:.texi=.aux) $(<:.texi=.log) $(<:.texi=.toc)

#.texi{html}.html:
#	$(silent)$(texi2html) $(texi2html_flags) $< 

.texi.html:
	-$(silent)$(texi2html) $(texi2html_flags) $< -o $@

.texi.pdf:
	-@echo $(verbose) & echo "Compiling $@ out of $<" & ( \
	pdfetex $(texflags) $< & \
	pdfetex $(texflags) $< & \
	$(texindex) $(<:.texi=.??) & \
	pdfetex $(texflags) $< )
	-@$(del) $(<:.texi=.aux) $(<:.texi=.log) $(<:.texi=.toc)

.txi.info:
	-$(makeinfo) $(makeinfo_flags) $< -o $@

.txi.dvi:
#	$(TEXI2DVI) $(TEXI2DVI_FLAGS) $<
	-etex $(texflags) $<
	-etex $(texflags) $<
	-$(texindex) $(<:.txi=.??)
	-etex $(texflags) $<
	$(del) $(<:.txi=.aux) $(<:.txi=.log) $(<:.txi=.toc)

#.txi{html}.html:
#	$(texi2html) $(texi2html_flags) $< 

.txi.html:
	-$(texi2html) $(texi2html_flags) $< -o $@

.txi.pdf:
	-pdfetex $(texflags) $<
	-pdfetex $(texflags) $<
	-$(texindex) $(<:.txi=.??)
	-pdfetex $(texflags) $<
	-@$(del) $(<:.txi=.aux) $(<:.txi=.log) $(<:.txi=.toc)

.tex.pdf:
	-pdflatex $(texflags) $<
	-pdflatex $(texflags) $<

#.tex{html}.html:
#	-if NOT exist "$(htmldocdir)" mkdir "$(htmldocdir)"
#	-htlatex --force --output-dir html $<

.tex.html:
	-if NOT exist "$(htmldocdir)" mkdir "$(htmldocdir)"
	-htlatex --force $<

.dvi.ps:
	-$(dvips) $(dvipsflags) $< -o

$(makeinfo):
	pushd $(texk_srcdir)\texinfo & $(make) all & popd

$(install_info):
	pushd $(texk_srcdir)\texinfo & $(make) all & popd

doc: $(infofiles) $(htmldocfiles) $(pdfdocfiles)

installdirs = $(installdirs) "$(infodir)" "$(docdir)" "$(htmldocdir)" "$(pdfdocdir)"

# install:: install-info install-doc

install-info:: $(install_info) $(infofiles)
!if !exist("$(infodir)\dir") && exist($(texk_srcdir)\dir-example)
	if NOT EXIST "$(infodir)\dir" \
	    $(copy) $(texk_srcdir)\dir-example "$(infodir)\dir"
!endif
	-@echo $(verbose) & ( \
		for %i in ($(infofiles)) do ( \
			$(copy) %i "$(infodir)\%i" & \
			$(install_info) --version > nul & \
			if ERRORLEVEL 0 \
	 			$(install_info) --info-dir="$(infodir)" $(installinfoflags) "$(infodir)/%i" \
		) \
	)

install-doc:: $(pdfdocfiles) $(htmldocfiles)
	-@echo $(verbose) & ( \
		if "$(pdfdocfiles)" neq "" ( \
			for %i in ($(pdfdocfiles)) do ( \
				$(copy) %i "$(pdfdocdir)\%i" $(redir_stdout) ) ) \
	)
	-@echo $(verbose) & ( \
		if "$(htmldocfiles)" neq "" ( \
			for %i in ($(htmldocfiles)) do ( \
				$(copy) %i "$(htmldocdir)\%i" $(redir_stdout) ) ) \
	)

extraclean::
	-@echo $(verbose) & ( \
			for %i in (cm cp fl fn ky op pg toc tp vr) do $(del) *.%i *.%is $(redir_stderr) \
		)

distclean::
	-@echo $(verbose) & ( \
		for %i in ($(infofiles)) do $(del) %i $(redir_stderr) \
	)
	-@echo $(verbose) & ( \
		$(deldir) html $(redir_stderr)  & \
		for %i in ($(htmldocfiles)) do $(del) %i $(redir_stderr) \
	)
	-@echo $(verbose) & ( \
		for %i in ($(pdfdocfiles)) do $(del) %i $(redir_stderr) \
	)
	-@$(del) *.info-? *.info-??

!else
#
# Do nothing if no info files
#
doc:

install-info::

install-doc::

!endif

# End of texi.mak.
#
# Local Variables:
# mode: Makefile
# End:
