
require "newgetopt.pl";
use lib qw (. ./msvc);
use FileUtils qw(canon_dir cleandir make_link newpath 
		  normalize substitute_var_val dirname
		  rec_rmdir sync_dir walk_dir start_redirection
		  stop_redirection sync_dir);

local $opt_root_dir = "c:/source/XEmTeX";
local $opt_dest_dir = "c:/source/TeXLive/Master/source";

%include = (
		 'texlive' => [ "graphics/bmeps",
				"graphics/bmp2png",
				"graphics/gd",
				"graphics/jpeg",
				"graphics/jpeg2ps",
				"graphics/libpng",
				"graphics/tiff",
				"graphics/tiff2png",
				"graphics/xpm",
				"graphics/win32.mak",
				"gnu/bzip2",
				"gnu/gzip",
				"gnu/win32.mak",
				"libs/expat",
				"libs/freetype",
				"libs/freetype2",
				"libs/geturl",
				"libs/libgnuw32",
				"libs/libgsw32",
				"libs/libttf",
				"libs/md5",
				"libs/regex",
				"libs/win32.mak",
				"libs/xpdf",
				"libs/zlib",
				"msvc",
				"setup",
				"texk/afm2pl",
				"texk/aleph",
				"texk/bibtex",
				"texk/bibtex8",
				"texk/chktex",
				"texk/cjkutils",
				"texk/context",
				"texk/ctie",
				"texk/cweb",
				"texk/detex",
				"texk/devnag",
				"texk/doc",
				"texk/dtl",
				"texk/dvi2tty",
				"texk/dvicopy",
				"texk/dvidvi",
				"texk/dviljk",
				"texk/dvipdfm",
				"texk/dvipdfmx",
				"texk/dvipng",
				"texk/dvipos",
				"texk/dvipsk",
				"texk/etex",
				"texk/fontware",
				"texk/gsftopk",
				"texk/kpathsea",
				"texk/lacheck",
				"texk/latexmk",
				"texk/lcdf-typetools",
				"texk/makeindexk",
				"texk/mf",
				"texk/mfware",
				"texk/mp",
				"texk/mpware",
				"texk/musixflx",
				"texk/odvipsk",
				"texk/omega",
				"texk/omegafonts",
				"texk/omegaware",
				"texk/otps",
				"texk/owindvi",
				"texk/patgen",
				"texk/pdfetex",
				"texk/pdftex",
				"texk/pdftexdir",
				"texk/pdfxtex",
				"texk/ps2eps",
				"texk/ps2pkm",
				"texk/psutils",
				"texk/seetexk",
				"texk/t1utils",
				"texk/tests",
				"texk/tex",
				"texk/tex4htk",
				"texk/texdoctk",
				"texk/texi2html",
				"texk/texinfo",
				"texk/texmfmp",
				"texk/texware",
				"texk/tie",
				"texk/triptrap",
				"texk/ttf2pfb",
				"texk/ttf2pk",
				"texk/ttf2pt1",
				"texk/ttfdump",
				"texk/vlna",
				"texk/web",
				"texk/web2c",
				"texk/win32.mak",
				"texk/windvi",
				"texk/xemtex" ],
		 'xemtex' => [ "graphics/bmeps",
			       "graphics/bmp2png",
			       "graphics/gd",
			       "graphics/jpeg",
			       "graphics/jpeg2ps",
			       "graphics/libpng",
			       "graphics/tiff",
			       "graphics/tiff2png",
			       "graphics/xpm",
			       "graphics/win32.mak",
			       "gnu/bzip2",
			       "gnu/compface",
			       "gnu/diffutils",
			       "gnu/grep",
			       "gnu/gzip",
			       "gnu/ispell",
			       "gnu/sed",
			       "gnu/setargv",
			       "gnu/wget",
			       "gnu/win32.mak",
			       "gstools/fonts",
			       "gstools/ghostscript",
			       "gstools/gsview",
			       "gstools/pstoedit",
			       "gstools/win32.mak",
			       "libs/expat",
			       "libs/freetype",
			       "libs/freetype2",
			       "libs/geturl",
			       "libs/libgnuw32",
			       "libs/libgsw32",
			       "libs/libttf",
			       "libs/md5",
			       "libs/regex",
			       "libs/unzip",
			       "libs/win32.mak",
			       "libs/xpdf",
			       "libs/zlib",
			       "msvc",
			       "perl/expat-1.95.6",
			       "perl/libwin32-0.191",
			       "perl/libwww-perl-5.69",
			       "perl/libxml-perl-0.07",
			       "perl/perl-5.8.0",
			       "perl/perlref-5.004.1",
			       "perl/tcl8.3.4",
			       "perl/tk8.3.4",
			       "perl/Tk800.024",
			       "perl/Win32-API-0.41",
			       "perl/win32.mak",
			       "perl/Win32API-Registry-0.21",
			       "perl/XML-DOM-1.42",
			       "perl/XML-Parser-2.31",
			       "perl/XML-RegExp-0.03",
			       "perl/XML-RSS-0.7",
			       "setup",
			       "texk/afm2pl",
			       "texk/aleph",
			       "texk/bibtex8",
			       "texk/context",
			       "texk/ctie",
			       "texk/cweb",
			       "texk/dvipdfm",
			       "texk/dvipdfmx",
			       "texk/dvipng",
			       "texk/dvipos",
			       "texk/dvipsk",
			       "texk/etex",
			       "texk/gsftopk",
			       "texk/kpathsea",
			       "texk/lcdf-typetools",
			       "texk/makeindexk",
			       "texk/mp",
			       "texk/mpware",
			       "texk/musixflx",
			       "texk/otps",
			       "texk/pdfetex",
			       "texk/pdftex",
			       "texk/pdftexdir",
			       "texk/pdfxtex",
			       "texk/ps2eps",
			       "texk/t1utils",
			       "texk/tests",
			       "texk/tex",
			       "texk/tex4htk",
			       "texk/texmfmp",
			       "texk/texware",
			       "texk/tie",
			       "texk/triptrap",
			       "texk/web",
			       "texk/web2c",
			       "texk/win32.mak",
			       "texk/xemtex",
			       "xemacs/elisp",
			       "xemacs/texmf",
			       "xemacs/win32.mak",
			       "xemacs/xemacs-21.5" ],
		 'xemacs' => [ 
			       "graphics/jpeg",
			       "graphics/libpng",
			       "graphics/tiff",
			       "graphics/xpm",
			       "graphics/win32.mak",
			       "gnu/bzip2",
			       "gnu/compface",
			       "gnu/diffutils",
			       "gnu/grep",
			       "gnu/gzip",
			       "gnu/ispell",
			       "gnu/sed",
			       "gnu/setargv",
			       "gnu/wget",
			       "gnu/win32.mak",
			       "libs/libgnuw32",
			       "libs/md5",
			       "libs/regex",
			       "libs/win32.mak",
			       "libs/zlib",
			       "msvc",
			       "setup",
			       "xemacs/elisp",
			       "xemacs/texmf",
			       "xemacs/win32.mak",
			       "xemacs/xemacs-21.5" ]
		);

@exclude = (
	    "libs/expat/",
	    "libs/geturl-new/",
	    "libs/freetype/",
	    "graphics/netpbm/",
	    "graphics/tiff/",
	    "setup/backup/",
	    "texk/dvipdfm/config\\.h",
	    "texk/dvipdfm/latex-support/",
	    "texk/dvipsk/dvips\\.info.*",
	    "texk/kpathsea-fp/",
	    "texk/kpathsea/kpathsea\\.cps",
	    "texk/kpathsea/kpathsea\\.info.*",
	    "texk/kpse",
	    "texk/kpse-shared.*",
	    "texk/kpse-shared/",
	    "texk/kpsexx/",
	    "texk/tetex/latex\\.info.*",
	    "texk/ttfdump/docs/ttfdump\\.dvi",
	    "texk/ttfdump/docs/ttfdump\\.html",
	    "texk/tth/",
	    "texk/doc/web2c\\.cps",
	    "texk/doc/web2c\\.info.*",
	    "texk/enctex/",
	    "texk/eomegadir-1\\.23/",
	    "texk/eomegadir-1\\.15/",
	    "texk/eomegadir-new/",
	    "texk/eotrip/",
	    "texk/windvi-0\\.68/",
	    "texk/gsftopk-new/",
	    "texk/windvi-new/",
	    "texk/texi2html-1\\.64/",
	    "texk/dvipng-0\\.2/",
	    "texk/dvipng-1\\.0/",
	    "texk/.*-beta/",
	    "backup/",
	    ".*~",
	    "Makefile\\.in",
	    "configure",
	    "configure\\.in",
	    "config\\.guess",
	    "config\\.sub",
	    ".*/Makefile\\.in",
	    ".*/configure",
	    ".*/configure\\.in",
	    ".*/config\\.guess",
	    ".*/config\\.sub",
	    ".*\\.dll",
	    ".*\\.exe",
	    ".*\\.mk",
	    ".*\\.obj",
	    ".*\\.orig",
	    ".*\\.pdb",
	    ".*\\.rej",
	    ".*/\\.cvsignore",
	    ".*/CVS/",
	    ".*/diffs/",
	    ".*/dynamic-debug/",
	    ".*/dynamic/",
	    ".*/static-debug/",
	    ".*/static/"
	   );

my $exclude_regexp = "(" . join("|", @exclude) . ")" ;

# print "${exclude_regexp}\n";

&main;

exit(0);

sub process {
  my ($dir, @l) = @_;

  @l = grep { "$dir/$_" !~ m/${exclude_regexp}/ } @l;

  map { if (-f "$dir/$_") { print ZIP "$dir/$_\n"; } } @l;
}

sub prune {
  my ($dir) = @_;

  return ($dir =~ m/${exclude_regexp}/);
}

sub main {

  unless (&NGetOpt (
		    "dist=s",
		    "root_dir=s"))
    {
	print STDERR "Try `configure --help'";
	exit 1;
    }

  chdir($opt_root_dir);

  unlink("${opt_dest_dir}/${opt_dist}-source.zip");
  open ZIP, "|zip -9\@or ${opt_dest_dir}/${opt_dist}-source.zip";
  for my ${file} (@{${include}{$opt_dist}}) {
    &FileUtils::walk_dir(${file}, \&process, \&prune);
  }
  close ZIP;
}
