/*
  Purpose:
  Author: Fabrice Popineau <Fabrice.Popineau@supelec.fr>
  Date:

  Compile with : cl FinalizeXEmTeX.c msi.lib kernel32.lib /link /nodefaultlib /entry:DllMain /DLL
*/

#undef WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <msi.h>
#include <msiquery.h>
#include <stdio.h>

#include "schemes.h"

#undef DEBUG
/* #define DEBUG 1 */

char *lstrdup(const char *s)
{
    char *p = HeapAlloc(GetProcessHeap(), 0, lstrlen(s)+1);
    lstrcpy(p, s);
    return p;
}

void lfree(char *s)
{
    HeapFree(GetProcessHeap(), 0, s);
}

char *lstrchr(char *p, char c)
{
    if (p != NULL) {
	while (*p != '\0' && *p != c) p++;
	if (*p == '\0') p = NULL;
    }
    return p;
}

#if 0
char *lstrrchr(char *p, char c)
{
    if (p != NULL) {
	char *q = p + lstrlen(p) - 1;
	while (q >= p && *q != c) q--;
	if (q < p) p = NULL;
    }
    return p;
}
#endif

BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
    return TRUE;
}

#define IS_DIR_SEP(c) (c == '\\')
#define S_ISDIR(m) (((m) & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY)

/*
  CreateDirectoryPath
  Create a directory path (borrowed from GNU File Utilities).
*/

static int ending = -1;

BOOL __stdcall CreateDirectoryPath (LPCTSTR path)
{
    char *dirpath = lstrdup(path);
    BOOL ret = TRUE;
    DWORD fa;
    char *p;
    char * cp;

    ending = -1;

    if (! dirpath)
	return FALSE;

    // No more slashes
    for (p = dirpath; p && *p; p++)
	*p = (*p == '/' ? '\\' : *p);

    if ((fa = GetFileAttributes(dirpath)) == -1) {
	char * slash = dirpath;
	if (slash[0] && slash[1] == ':')
	    slash += 2;
	else if (IS_DIR_SEP(slash[0]) && slash[1] == slash[0]) {
	    // UNC file name
	    slash += 2;
	    while (*slash && !IS_DIR_SEP(*slash))
		slash++;
	    if (IS_DIR_SEP(*slash)) {
		slash++;
		while (*slash && ! IS_DIR_SEP(*slash))
		    slash++;
	    }
	}
	while (IS_DIR_SEP(*slash))
	    slash++;
	while ((cp = lstrchr(slash, '/')) || (cp = lstrchr(slash, '\\'))) {
	    slash = cp;
	    *slash = '\0';
	    if ((fa = GetFileAttributes(dirpath)) == -1) {
		if (ending < 0) ending = slash - dirpath;
		if (CreateDirectory(dirpath, NULL) == 0) {
		    ret = FALSE;
		    goto done;
		}
	    }
	    else if (! S_ISDIR(fa)){
		ret = TRUE;
		goto done;
	    }
	    *slash++ = '\\';
	    while (IS_DIR_SEP(*slash))
		slash++;
	}
	if (CreateDirectory(dirpath, NULL) == 0) {
	    ret = FALSE;
	    goto done;
	}
    }
    else {
	if (! S_ISDIR(fa)) {
	    ret = FALSE;
	    goto done;
	}
    }
 done:
    lfree(dirpath);
    return ret;
}

BOOL __stdcall RemoveDirectoryPath (LPCTSTR path)
{
    char *dirpath = lstrdup(path);
    BOOL ret = TRUE;
    char previous;
    char *cp;

    if (ending > -1) {
	char *slash = dirpath + lstrlen(dirpath);
	while (slash >= dirpath + ending) {
	    *slash = '\0';
	    RemoveDirectory(dirpath);
	    while (IS_DIR_SEP(*--slash));
	    while (! IS_DIR_SEP(*slash)) slash--;
	}
    }
    lfree(dirpath);
    return ret;
}

#undef S_ISDIR

__declspec(dllexport) UINT __stdcall FixLongToShortPathName(MSIHANDLE hModule)
{
    char szPathBuf[2*_MAX_PATH];
    char szInstallDirBuf[2*_MAX_PATH];
    DWORD szInstallDirLength = sizeof(szInstallDirBuf);
    UINT uiRet =  MsiGetTargetPath(hModule, "INSTALLDIR", szInstallDirBuf, &szInstallDirLength);

    if (uiRet != ERROR_SUCCESS) {
	return -1;
    }

#ifdef DEBUG
    MessageBox(NULL, szInstallDirBuf, "INSTALLDIR", MB_OK);
#endif

    /* We need to create it if it doesn't exist */
    if (CreateDirectoryPath(szInstallDirBuf) == FALSE) {
	return -1;
    }
#ifdef DEBUG
    MessageBox(NULL, szInstallDirBuf, "CreateDirectory", MB_OK);
#endif

    if (GetShortPathName(szInstallDirBuf, szPathBuf, sizeof(szPathBuf)) == 0) {
	return -1;
    }
#ifdef DEBUG
    MessageBox(NULL, szPathBuf, "Short Path", MB_OK);
#endif
    
    RemoveDirectory(szInstallDirBuf);

#ifdef DEBUG
    MessageBox(NULL, szPathBuf, "Remove Directory", MB_OK);
#endif

    uiRet = MsiSetProperty(hModule, "SHORTINSTALLDIR", szPathBuf);

#ifdef DEBUG
    MessageBox(NULL, szPathBuf, "Set Property", MB_OK);
#endif

    if (uiRet != ERROR_SUCCESS) {
	return -1;
    }

#ifdef DEBUG
    MessageBox(NULL, szPathBuf, "Done", MB_OK);
#endif

    return 0;
}

UINT __stdcall Execute(LPTSTR cmdline)
{
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    int i;

    for (i = 0; i < sizeof(si); i++) ((char *)&si)[i] = '\0';
    si.cb = sizeof (si);
    si.dwFlags = STARTF_USESHOWWINDOW;
    si.wShowWindow = SW_HIDE;
    for (i = 0; i < sizeof(pi); i++) ((char *)&pi)[i] = '\0';
    if (!CreateProcess (NULL, (LPTSTR)cmdline, NULL, NULL, FALSE, 0,
			NULL, NULL, &si, &pi))
    {
        /* error("Could not start process %s", prog); */
	return 1;
    }
    /* Wait for the process to enter an idle state */
    WaitForSingleObject (pi.hProcess, INFINITE);

    /* Close the handles */
    CloseHandle (pi.hThread);
    CloseHandle (pi.hProcess);

    return 0;
}

__declspec(dllexport) UINT __stdcall Finalize(MSIHANDLE hModule) 
{
    char szDirBuf[2 * _MAX_PATH];
    char szPathBuf[2 * _MAX_PATH];
    char szInstallDirBuf[2 * _MAX_PATH];
    DWORD szInstallDirLength = sizeof(szInstallDirBuf);
    UINT uiRet =  MsiGetTargetPath(hModule, "INSTALLDIR", szInstallDirBuf, &szInstallDirLength);

    if (uiRet != ERROR_SUCCESS) {
	return -1;
    }

    szInstallDirLength = lstrlen(szInstallDirBuf);
    if (szInstallDirBuf[szInstallDirLength - 1] != '\\') {
	szInstallDirBuf[szInstallDirLength] = '\\';
	szInstallDirLength++;
	szInstallDirBuf[szInstallDirLength] = '\0';
    }

    lstrcpy(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "bin\\win32;");

    /* Need to set PATH : fmtutil calls external programs ! */
    GetEnvironmentVariable("PATH", szPathBuf, sizeof(szPathBuf));

    lstrcat(szDirBuf, szPathBuf);
    SetEnvironmentVariable("PATH", szDirBuf);

#if 0
    lstrcpy(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "texmf-var\\web2c");
    SetEnvironmentVariable("TEXMFCNF", szDirBuf);
#endif

    lstrcpy(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "temp");
    SetEnvironmentVariable("TEXMFTEMP", szDirBuf);

    lstrcpy(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "texmf");
    SetEnvironmentVariable("TEXMFMAIN", szDirBuf);

    lstrcpy(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "xemtex\\perl\\site\\lib;");
    lstrcat(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "xemtex\\perl\\lib");
    SetEnvironmentVariable("PERL5LIB", szDirBuf);

    /* Be safe, remove anything dangerous */
    SetEnvironmentVariable("TEXMF", NULL);
    SetEnvironmentVariable("TEXMFLOCAL", NULL);
    SetEnvironmentVariable("TEXMFEXTRA", NULL);

#if 0
    SetEnvironmentVariable("KPATHSEA_DEBUG", "-1");
    SetEnvironmentVariable("KPATHSEA_DEBUG_OUTPUT", "c:\\err.log");
#endif
    /* Call mktexlsr */
    lstrcpy(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "bin\\win32\\mktexlsr");
    Execute(szDirBuf);

    /* Call fmtutil */
    lstrcpy(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "bin\\win32\\fmtutil --all --dolinks --force");
    Execute(szDirBuf);

    /* Call updmap */
    lstrcpy(szDirBuf, szInstallDirBuf);
    lstrcat(szDirBuf, "bin\\win32\\updmap");
    Execute(szDirBuf);

    return 0;
}

#ifdef DEBUG

static void CheckError(UINT err, char *s1, char *s2)
{
    switch (err) {
    case ERROR_SUCCESS:
	break;
    case ERROR_ACCESS_DENIED:
	MessageBox(NULL, s1, "Access was not permitted.", MB_ICONERROR);
	break;
    case ERROR_FUNCTION_FAILED:
	MessageBox(NULL, s1, "The function failed.", MB_ICONERROR);
	break;
    case ERROR_INVALID_DATA:
	MessageBox(NULL, s1, "A validation was requested and the data did not pass. For more information, call MsiViewGetError.",
		   MB_ICONERROR);
	break;
    case ERROR_INVALID_HANDLE:
	MessageBox(NULL, s1, "An invalid or inactive handle was supplied.", MB_ICONERROR);
	break;
    case ERROR_INVALID_HANDLE_STATE:
	MessageBox(NULL, s1, "The handle is in an invalid state.", MB_ICONERROR);
	break;
    case ERROR_INVALID_PARAMETER:
	MessageBox(NULL, s1, "An invalid parameter was passed to the function.", MB_ICONERROR);
	break;
    }
}

#else

#define CheckError(x, s1, s2)

#endif

static void __stdcall ResetGlobalSelection(MSIHANDLE hModule)
{
    char sQuery[512], sBuffer[512];
    MSIHANDLE hwView, hwRecord;
    UINT errCode, uiSize = sizeof(sBuffer);
    char **p;
    MSIHANDLE hDatabase = MsiGetActiveDatabase(hModule); /* get handle for database */

    for (p = features_list; *p != NULL; p++) {
	errCode = MsiSetFeatureState(hModule, *p, INSTALLSTATE_ABSENT);
 	CheckError(errCode, "SetFeatureLevel", NULL);
    }
}

static void __stdcall ResetSelection(MSIHANDLE hModule, char *feature)
{
    char sQuery[512], sBuffer[512];
    MSIHANDLE hwView, hwRecord;
    UINT errCode, uiSize = sizeof(sBuffer);
    INSTALLSTATE iInstalled, iAction;

    errCode = MsiGetFeatureState(hModule, feature, &iInstalled, &iAction);
    CheckError(errCode, "GetFeatureLevel/ResetSelection", NULL);

    if (iAction != INSTALLSTATE_ABSENT) {
	errCode = MsiSetFeatureState(hModule, feature, INSTALLSTATE_ABSENT);
	CheckError(errCode, "SetFeatureLevel/ResetSelection", NULL);
    }
}

static void __stdcall SetSelection(MSIHANDLE hModule, char *feature)
{
    char sQuery[512], sBuffer[512];
    MSIHANDLE hwView, hwRecord;
    UINT errCode, uiSize = sizeof(sBuffer);
    INSTALLSTATE iInstalled, iAction;

    errCode = MsiGetFeatureState(hModule, feature, &iInstalled, &iAction);
    CheckError(errCode, "GetFeatureLevel", NULL);

    if (iAction != INSTALLSTATE_LOCAL) {
	errCode = MsiSetFeatureState(hModule, feature, INSTALLSTATE_LOCAL);
	CheckError(errCode, "SetFeatureLevel", NULL);
    }
}

__declspec(dllexport) UINT __stdcall PublishSchemeDescription(MSIHANDLE hModule) 
{
    char buffer[256];
    DWORD dwBufSize = sizeof(buffer);
    int i;
    UINT errCode;

    errCode = MsiGetProperty(hModule, "InstallScheme", buffer, &dwBufSize);
    CheckError(errCode, "SetProperty/PublishSchemeDescription", NULL);

    errCode = MsiSetProperty(hModule, "PreviousSelectScheme", buffer);
    CheckError(errCode, "SetProperty/PublishPreviousSelectScheme", NULL);

#ifdef DEBUG
    MessageBox(NULL, buffer, "Debug/PublishSchemeDescription", MB_OK);
#endif
    
    for (i = 0; schemes_list[i].scheme != NULL; i++) {
	if (lstrcmpi(buffer, schemes_list[i].scheme) == 0) {
	    MsiSetProperty(hModule, "SchemeDescription", schemes_list[i].description);
	    break;
	}
    }
    return ERROR_SUCCESS;
}

__declspec(dllexport) UINT __stdcall XEmTeXUpdateSelection(MSIHANDLE hModule) 
{
    char buffer[256];
    DWORD dwBufSize = sizeof(buffer);
    int i;
    char **from, **to, **from_features, **to_features;
    char sPreviousScheme[256];
    BOOL still_selected, was_selected;
    BOOL xemtex = FALSE;
    UINT errCode;
    MSIHANDLE hRecord;

    errCode = MsiGetProperty(hModule, "InstallXemtex", buffer, &dwBufSize);
    CheckError(errCode, "GetProperty/InstallXemtex", NULL);
#ifdef DEBUG
    MessageBox(NULL, buffer, "Debug/InstallXemtex", MB_OK);
#endif
    xemtex = lstrcmpi(buffer, "XEmTeX") == 0;
    if (! xemtex) {
	ResetSelection(hModule, "TLCorecollectionxemtex");
    }

    /* Get Property */
    dwBufSize = sizeof(sPreviousScheme);
    errCode = MsiGetProperty(hModule, "InstallPreviousScheme", sPreviousScheme, &dwBufSize);
    CheckError(errCode, "GetProperty/InstallPreviousScheme", NULL);

    dwBufSize = sizeof(buffer);
    errCode = MsiGetProperty(hModule, "InstallScheme", buffer, &dwBufSize);
    CheckError(errCode, "GetProperty/InstallScheme", NULL);
/* #ifdef DEBUG */
#if 0
    MessageBox(NULL, buffer, "Debug/InstallScheme New", MB_OK);
    MessageBox(NULL, sPreviousScheme, "Debug/InstallScheme Previous", MB_OK);
#endif

    if (lstrcmp(buffer, sPreviousScheme) == 0) {
	return ERROR_SUCCESS;
    }

#ifdef DEBUG
    MessageBox(NULL, buffer, "Updating Scheme", MB_OK);
#endif

#if 0
    hRecord = MsiCreateRecord(3);

/*     Installer is executing the installation script. Set up a */
/*     record specifying appropriate templates and text for */
/*     messages that will inform the user about what the custom */
/*     action is doing. Tell the installer to use this template and */
/*     text in progress messages. */
    MsiRecordSetString(hRecord, 0, NULL);
    MsiRecordSetString(hRecord, 1, "Updating Selection");
    MsiRecordSetString(hRecord, 2, "Installer is updating your package selection according to the current scheme");
    MsiRecordSetString(hRecord, 3, "[1] package [2]");
    MsiProcessMessage(hModule, INSTALLMESSAGE_ACTIONSTART, hRecord);
    
    MsiCloseHandle(hRecord);
#endif

    from_features= NULL;
    to_features = NULL;
    /* Find the right lines in the array */
    for (i = 0; schemes_list[i].scheme != NULL; i++) {
	if (!from_features && lstrcmpi(sPreviousScheme, schemes_list[i].scheme) == 0) {
	    from_features = schemes_list[i].features;
	}
	if (!to_features && lstrcmpi(buffer, schemes_list[i].scheme) == 0) {
	    to_features = schemes_list[i].features;
	}
    }

#if 0
    hRecord = MsiCreateRecord(2);
    MsiRecordSetString(hRecord, 0, NULL);
    /* Reset the ones that aren't selected anymore */
    MsiRecordSetString(hRecord, 1, "Resetting");
#endif
    for (from = from_features; *from != NULL; from++) {
	still_selected = FALSE;
	for (to = to_features; !still_selected && *to != NULL; to++) {
	    still_selected = (lstrcmpi(*from, *to) == 0);
	}
	if (!still_selected) {
	    ResetSelection(hModule, *from);
#if 0
	    MsiRecordSetString(hRecord, 2, *from);
	    MsiProcessMessage(hModule, INSTALLMESSAGE_ACTIONDATA, hRecord);
#endif
#ifdef DEBUG
		MessageBox(NULL, *from, "Selection Reset", MB_OK);
#endif
	}
    }

    /* Set the ones that are newly selected */
    MsiRecordSetString(hRecord, 1, "Resetting");
    for (to = to_features; *to != NULL; to++) {
	was_selected = FALSE;
	for (from = from_features; !was_selected && *from != NULL; from++) {
	    was_selected = (lstrcmpi(*from, *to) == 0);
	}
	if (!was_selected) {
	    SetSelection(hModule, *to);
#if 0
	    MsiRecordSetString(hRecord, 2, *from);
	    MsiProcessMessage(hModule, INSTALLMESSAGE_ACTIONDATA, hRecord);
#endif
#ifdef DEBUG
		MessageBox(NULL, *to, "Selection Set", MB_OK);
#endif
	}
    }

#if 0
    MsiCloseHandle(hRecord);
#endif

    if (xemtex) {
	SetSelection(hModule, "TLCorecollectionxemtex");
    }

    MsiSetProperty(hModule, "InstallPreviousScheme", buffer);

    return ERROR_SUCCESS;
}


#ifdef TEST

void main(int argc, char *argv[])
{

    char szPathBuf[2 * _MAX_PATH];
    char szInstallDirBuf[2 * _MAX_PATH];

    /* We need to create it if it doesn't exist */
    if (CreateDirectoryPath(argv[1]) == FALSE) {
	MessageBox(NULL, "Cannot create directory", "Error", MB_ICONERROR);
	return;
    }

    if (GetShortPathName(argv[1], szPathBuf, sizeof(szPathBuf)) == 0) {
	MessageBox(NULL, "Cannot get short path name", "Error", MB_ICONERROR);
	return;
    }
    
    RemoveDirectoryPath(argv[1]);

    MessageBox(NULL, szPathBuf, "Success", MB_OK);
}

#endif
