#
# MSI Tables Build tool for TeXLive
# Author: F. Popineau
# Date  : 27/03/2001
# Time-stamp: <03/05/16 05:59:44 popineau>
#

require "newgetopt.pl";
use lib ("c:/source/TeXLive/Master/Tools");
use FileUtils qw(canon_dir cleandir make_link newpath member
		 normalize substitute_var_val dirname diff_list remove_list
		 rec_rmdir sync_dir walk_dir start_redirection stop_redirection);
use Tpm;
use File::Basename;
use Cwd;

use Win32::API;

File::Basename::fileparse_set_fstype('unix');

use Strict;

my $FtpDir = "c:/Inetpub/ftp/fptex";
my $fpTeXVersion = "0.7";

local $opt_check = 0;
local $opt_tpm_dir = "${Tpm::TpmDir}";
# local $opt_master_dir = "${Tpm::MasterDir}";
local $opt_master_dir = "c:/Program Files/TeXLive";
local $opt_ftp_dir = "${FtpDir}/${fpTeXVersion}";
local $opt_arch;
local $opt_log = 0;
local $opt_debug = 0;
local $opt_verbose = 0;
local $short_master_dir = &GetShortPathName($opt_master_dir);


my @standalone_packages = (
			   "binary/bin-dvipdfm-static.tpm",
			   "binary/bin-dvipsk-static.tpm",
			   "binary/bin-gsftopk-static.tpm",
			   "binary/bin-ltx2rtf-static.tpm",
			   "binary/bin-omega-static.tpm",
			   "binary/bin-pdftex-static.tpm",
			   "binary/bin-tex4htk-static.tpm",
			   "binary/bin-windvi-static.tpm"
			  );

my $component_count = 0;
my $file_sequence = 1;
my %required_by = ();

my @guids = ();

my $tables = '';

&main;

1;

sub help {
  print "Usage: ${ARGV[0]} [options]\n";
  print "\n";
  print "  Support commands:\n";
  print "  --log=<file name>       log activity to file;\n";
  print "  --debug                 turn on debug trace;\n";
  print "  --verbose               some more messages\n";
  print "  --help                  displays this help text and exits\n";
}


local @master_files = ();

#
# Read the current list of guids for patterns
# Build the new pattern list
# Add new guids if needed
# Rewrite the list
#
sub prepare_guids {

}

sub join_raw {
  my ($sep, @l) = @_;
  my $r = "";
  while (@l) {
    $r .= shift @l;
    $r .= $sep if (@l);
  }
  return $r;
}

sub GetShortPathName {
  my $GetShortPathName = new Win32::API('Kernel32', 'GetShortPathName', 'PPN', 'N');
  if(not defined $GetShortPathName) {
    die "Can't import API GetShortPathName: $!\n";
  }
  my $lpBuffer = " " x 260;
  my $res = $GetShortPathName->Call($_[0], $lpBuffer, 260);
  return ($res == 0 ? '' : substr($lpBuffer, 0, $res));
}

sub get_component_from_file {
  my ($file) = @_;
  my $reg = "(^${file}\\||\\|${file}\$|^${file}\$)";
  print "reg = $reg\n";
  foreach my $key (keys %{$tables->{'File'}}) {
    print "matching $file with " . ${$tables->{'File'}->{$key}}[2] . "\n";
    if (${$tables->{'File'}->{$key}}[2] =~ m/$reg/xo) {
      return ${$tables->{'File'}->{$key}}[1];
    }
  }
  print STDERR "Couldn't find the component for file $file\n";
  return '';
}

sub get_feature_from_component {
  my ($component) = @_;
  foreach my $key (keys %{$tables->{'FeatureComponents'}}) {
    my $f = ${$tables->{'FeatureComponents'}->{$key}}[0];
    if (index($f,$component) > 0) {
      return $key;
    }
  }
  print STDERR "Couldn't find the feature for component $component\n";
  return '';
}

sub collect_files {
  local ($d, @lf) = @_;
  foreach my $f (@lf) {
    my $path = "$d/$f";
    if (-f "$path") {
      $path =~ s@^${opt_master_dir}/@@;
      $path =~ s@\\@/@g;
      if ($path !~ m@^texmf/tpm/lists/@
	 && $path !~ m@^bin/win32-static/@) {
	push @master_files, $path;
      }
    }
  }
}

# Returns the directory key associated with $dir in the Directory Table
# Create it if it does not exist.
sub dir_key {
  my ($dir) = @_;
  my @path = split ("/", "$dir");
  my $shortdir = &GetShortPathName("$opt_master_dir/$dir");
  $shortdir =~ s@^${short_master_dir}/@@;
#  print "short_master_dir = $short_master_dir\nshortdir = $shortdir\n";
  my @shortpath = split ("/", "$shortdir");
  my $key = "";
  my $parent = "INSTALLDIR";
#  print "path = @path\n";
  while (@path) {
    my $dir = shift @path;
    my $keydir = $dir;
    $keydir =~ s/[_-]//g;
    my $shortdir = shift @shortpath;
    $key = "$key$keydir";
#    print "key = $key\n";
#    print "foo = " . $tables->{'Directory'}->{$key} . "\n";
    if ($tables->{'Directory'}->{$key} eq undef) {
      if ($shortdir ne $dir) {
	$dir = "$shortdir|$dir";
      }
      @{$tables->{'Directory'}->{$key}} = ($key, $parent, "$dir");
    }
    $parent = $key;
  }
#  my @l = keys %tables; print "l = @l\n";
  return $key;
}

#
#
sub do_pattern {
  my ($feature, $condition, @patterns) = @_;
#  print "feature = $feature, cond = $condition\n";
  map {
    my $pattern = $_;
    my $component = "$feature$component_count";
#    print "pattern = $pattern\ncomponent = $component\n";
    if ($tables->{'Component'}->{$component}) {
      print STDERR "Error: component $component already exists !\n";
    }
    my $directory = &dir_key(&FileUtils::dirname($pattern));
    my $component_id = '';
    if ($pattern =~ m/\.(exe|dll)$/) {
      $component_id = $guids[$component_count];
    }
    my $keypath = '';
    
    # delay creation of the component because we need a keypath
    
    my @files = &FileUtils::globexpand(0, "$opt_master_dir/$pattern");
#    print "files = @files\n";
    map {
      my $file = "file$file_sequence"; # &FileUtils::basename($_);
      $keypath = $file if (! $keypath);
      my $filename = &FileUtils::basename($_);
      my $shortfile = &FileUtils::basename(&GetShortPathName($_));
      my @st = stat($_);
      my $filesize = $st[7];
      my $version = '';
      my $language = '';
      if ($_ =~ m/\.(exe|dll)$/) {
	($version, $language) = split ("\n", `getversion "$_"`);

	$version =~ s/,/./g;
#	print "version($_) = $version\n";
      }
      my $attributes = ($filename =~ /(\.exe|\.dll)$/ ? 1024 : 0);
      $filename = "$shortfile|$filename" if ($filename ne $shortfile);
      @{$tables->{'File'}->{$file}} = ($file, $component, $filename, $filesize, $version, $language, $attributes, $file_sequence);
      $file_sequence++;
    } @files;

    @{$tables->{'Component'}->{$component}} = ($component, $component_id, $directory, 0, $condition, $keypath);
    $component_count++;

  } @patterns;
}

sub do_feature {
  my ($tpmname) = @_;
  my $tpm = Tpm->new($tpmname);
  my $feature = $tpm->getAttribute('Provides');
  $feature =~ s@[/-]@@g;
#  print "feature = $feature\n";
  # Add the feature and the components
  my $feature_parent;
  my $title = $tpm->getAttribute('Title');
  $title =~ s/\n//gxo;
  if (length($title) > 64) {
    $title =~ s/\. .*$//;
  }
  if (length($title) > 64) {
    $title =~ s/[ \t][^ \t]*$//;
  }
  if ($title eq '' || length($title) > 64) {
    print STDERR "Cutting title for $feature from $title to ";
    $title = $tpm->getAttribute('Name');
    print STDERR "$title\n";
  }
  my $description = substr ($tpm->getAttribute('Description'), 0, 255);
  $description =~ s/\n//gxo;
  my $display = 2;		# collapsed
  my $level = 3;		# see InstallLevel
  my $directory = '';
  my $attributes = 0;

  # If it is a collection
  if ($tpm->getAttribute('Type') eq 'collection') {
    $feature_parent = '';
  }
  # or if it is a package or binary
  else {
    $feature_parent = "&get_feature_parent(\"$feature\")";
  }
  my %req = $tpm->getHash('Requires');
  foreach my $subdir (keys %req) {
    map { 
      $_ = "$subdir$_";
      $_ =~ s@-@@g;
#      print "$subdir/$_\n"; 
      push @{$required_by{$_}}, $feature 
    } @{$req{$subdir}}
  }
  if ($tables->{'Feature'}->{$feature}) {
    print STDERR "Error: feature $feature already exists!\n";
  }
  @{$tables->{'Feature'}->{$feature}} = ($feature, $feature_parent, $title, $description, $display, $level, $directory, $attributes);

  # Create one component per line of patterns
  # Regular components
  my $start_component = $component_count;
  my @patterns = $tpm->getList("BinPatterns");
  @patterns = grep { $_ !~ m/ARCH/ } @patterns;
  push @patterns, $tpm->getList("RunPatterns");
  &do_pattern($feature, '', @patterns); 
  # Doc components
  @patterns = $tpm->getList("DocPatterns");
  &do_pattern($feature, 'DOC', @patterns); 
  # Source components
  @patterns = $tpm->getList("SourcePatterns");
  &do_pattern($feature, 'SOURCE', @patterns); 

  # List components in the feature
  my $feature_components = "";
  for (my $i = $start_component; $i < $component_count; $i++) {
    # we put the raw line here because there will be multiple entries with the same key
    $feature_components .= "$feature\t$feature$i\n";
  }
  @{$tables->{'FeatureComponents'}->{$feature}} = ($feature_components);
  # Add files

}

sub do_subdir {
  local ($subdir, $fn) = @_;
  print " Directory $subdir\n" if ($opt_verbose);
  my $count = 0;
  while (<$subdir/*>) {
#    next if ($count++ > 3);
    my $tpmname = $_;
    next if ($tpmname =~ m/~$/);
    next if ($tpmname =~ m/-static/ && ! $opt_standalone);
    print "Doing $tpmname\n" if ($opt_verbose);
    &{$fn}("$tpmname");
  }
}

sub get_feature_parent {
  my ($f) = @_;
  my $parent = ${$required_by{$f}}[0];
  $parent =~ s@^.*/([^/]*)$@\1@;
  $parent=~ s/[_-]//g;
  print "get_feature_parent($f) = $parent\n";
  return $parent;
}

sub main {

  unless (&NGetOpt (
		    "msi=s",
		    "sync",
		    "log=s",
		    "verbose", 
                    "debug", "help"))
    {
      print STDERR "Try `$0 --help'";
      exit 1;
    }

  if ($opt_help) {
    &help;
    exit 0;
  }

  if ($opt_sync) {
    local ($f1, $f2) = ("c:/source/texlive/Master",
			"c:/Program Files/TeXLive");
    for my $subdir ("texmf", "FAQ", "usergrps", "source", "man", "info") {
      &sync_dir("$f1/$subdir", "$f2/$subdir", "", "", 0, 0, 0);
    }
  }

  # read the GUID
  open GUID, "<tl-guids.txt";
  @guids = <GUID>;
  close GUID;
  chomp @guids;

  #  &walk_dir("$opt_masterdir", &collect_files);
  #  my $file_tree = &FileUtils::build_tree(@master_files);

  &do_subdir("$opt_tpm_dir/binary", \&do_feature);
  &do_subdir("$opt_tpm_dir/package", \&do_feature);
  &do_subdir("$opt_tpm_dir/support", \&do_feature);
  &do_subdir("$opt_tpm_dir/collection", \&do_feature);

  # Fix the FeatureParent field. Make only collections and their
  # direct childs appear. Other features disappear.
  # Add the Condition table.
  foreach my $k (keys %{$tables->{'Feature'}}) {
#    print "feature $k " . ${$tables->{'Feature'}->{$k}}[1] . "\n";
    ${$tables->{'Feature'}->{$k}}[1] = eval (${$tables->{'Feature'}->{$k}}[1]);
#    print "feature $k " . ${$tables->{'Feature'}->{$k}}[1] . "\n";
  }
  #  &FileUtils::walk_tree($file_tree, do_tables);


  # Condition table
  foreach my $key (keys %{$tables->{'Feature'}}) {
    if (@{$required_by{$key}}) {
      my $condition = join(" OR ", map { $_ = "&" . $_ . "=3"} @{$required_by{$key}});
      print "$key\t3\t$condition\n";
      @{$tables->{'Condition'}->{$key}} = ($key, 3, $condition);
    }
  }

  # Add shortcuts
  my $windvi_component = &get_component_from_file('windvi.exe');
  my $windvi_feature = &get_feature_from_component($windvi_component);
  my $winclient_component = &get_component_from_file('winclient.exe');
  my $winclient_feature = &get_feature_from_component($winclient_component);

  @{$tables->{'Shortcut'}->{'windvi'}} = ('windvi.exe', 'ProgramMenuFolder', 'windvi.exe', $windvi_component, $windvi_feature, '-single', 'DVI file viewer', 0, 'IconBF249F792.exe', 1, 1, '');
  @{$tables->{'Shortcut'}->{'fpTeXHome'}} = ('fpTeXHome', 'ProgramMenuFolder', 'FPTEX\'~1|fpTeX\'s Home', 'TeXLive', '[SHORTCUTPATH]', '', '', 0, '', 0, 1, '');
  @{$tables->{'Shortcut'}->{'TeXLiveHTML'}} = ('TeXLiveHTML', 'ProgramMenuFolder', 'TEXLIV~1|TeXLive Documentation', 'file26376', 'tldocenglish', '', '', 0, '', 0, 1, '');
  @{$tables->{'Shortcut'}->{'Winclient'}} = ('Winclient', 'ProgramMenuFolder', 'Winclient', $winclient_component, $winclient_feature, '', '', 0, 'IconBF249F793.exe', 1, 1, '');

  # Add extensions
  @{$tables->{'Extension'}->{'dvi'}} = ('dvi', $windvi_component, 'windvi.Document', '', $windvi_feature);
  @{$tables->{'Extension'}->{'tex'}} = ('tex', $winclient_component, 'tex.Document', '', $winclient_feature);
  @{$tables->{'Extension'}->{'bib'}} = ('bib', $winclient_component, 'bib.Document', '', $winclient_feature);

  # Add ProgIds
  @{$tables->{'ProgId'}->{'windvi.Document'}} = ('windvi.Document', '', '', 'DVI Document', 'IconBF249F792.exe', 1);
  @{$tables->{'ProgId'}->{'tex.Document'}} = ('tex.Document', '', '', 'TeX Document', 'IconBF249F793.exe', 1);
  @{$tables->{'ProgId'}->{'bib.Document'}} = ('bib.Document', '', '', 'BibTeX Document', 'IconBF249F793.exe', 1);

  # Add Media
  @{$tables->{'Media'}->{'1'}} = (1, 100000, 'TeXLive CDRom', '', 'TEXLIVE8', '');

  # Export tables
  # $cmd = "msidb -e -d$opt_msi -fc:\source\fptex\mswin32\MSI " . join(" ", (keys %{$tables}));
  # system($cmd);

  # Rewrite tables
  foreach my $t (keys %{$tables}) {
    # copy template into current directory
    my $ft = substr($t, 0, 8) . ".idt";
    &FileUtils::copy("Template/$t.idt", $ft);
    # add the lines
    open IDT, ">>$ft";
    binmode(IDT);
    print "Table $t:\n";
    foreach my $k (sort(keys %{$tables->{$t}})) {
      my $line = &join_raw("\t",  @{$tables->{$t}->{$k}});
      $line =~ s/\n*$/\n/x;
      print IDT $line;
    }
    close IDT
  }

  # Import tables
  $cmd = "msidb -i -d$opt_msi -fc:\\source\\fptex\\mswin32\\MSI " . join(" ", map { $_ = substr($_, 0, 8); } (keys %{$tables}));
  print "$cmd\n";
  system($cmd);
}
