#
# MSI Tables Build tool for TeXLive and XEmTeX
# Author: F. Popineau
# Date  : 27/03/2001
# Time-stamp: <04/11/16 16:50:17 popineau>
#

local $RootDir = "c:\\Source\\XEmTeX";

require "newgetopt.pl";
use lib ("c:\\Source\\XEmTeX\\msvc");
use FileUtils qw(canon_dir cleandir make_link newpath member is_absolute
		 normalize substitute_var_val dirname diff_list remove_list
		 rec_rmdir sync_dir walk_dir start_redirection stop_redirection);
use Tpm;
use File::Basename;
use File::Copy;
use Cwd;

use Win32::API;

File::Basename::fileparse_set_fstype('unix');

use Strict;

my $FtpDir = "c:/Inetpub/ftp/fptex";
my $fpTeXVersion = "0.7";

local $opt_check = 0;
local $opt_master_dir = "${Tpm::MasterDir}";
local $opt_ftp_dir = "${FtpDir}/${fpTeXVersion}";
local $opt_arch;
local $opt_log = 0;
local $opt_debug = 0;
local $opt_verbose = 0;
local $opt_target = '';
local $opt_help = '';
local $opt_sync = '';
local $opt_rebuild_cab = '';
local $opt_logdir = "c:/source/XEmTeX/tmp/logs";

my $tmpdir = "${RootDir}\\tmp\\setup";
my $short_master_dir = &GetShortPathName($opt_master_dir);

my $component_count = 0;
my $file_sequence = 1;
my %required_by = ();

my @guids_list = ();
my %guids_table = ();
my $guids_count = 0;

my $tables = '';

my %filemapping = ();
my $cab;

my $product = '';
my $product_version = '';

my $xemtex_version = "0.5";
my $xemacs_version = "21.5.18";
my $texlive_version = "2004";

# @cabfiles = ("file12", "file10", "file1", "file2");
#     @cabfiles = sort { my $n1 = substr($a, 4);
# 		       my $n2 = substr($b, 4);
# 		       print "n1 = $n1, n2 = $n2\n";
# 		       return $n2 < $n1; } @cabfiles;
# print "hello : " . (join " ", @cabfiles) . "\n";
# exit 0;

&main;

1;

sub help {
  print "Usage: ${ARGV[0]} [options]\n";
  print "\n";
  print "  Support commands:\n";
  print "  --target=(xemtex|xemtex-net|texlive|texlive-inst|texlive-demo)";
  print "  --log=<file name>       log activity to file;\n";
  print "  --debug                 turn on debug trace;\n";
  print "  --verbose               some more messages\n";
  print "  --help                  displays this help text and exits\n";
}


my @master_files = ();

sub join_raw {
  my ($sep, @l) = @_;
  my $r = "";
  while (@l) {
    my $e = shift @l;
    $e =~ s/\s*$//;
    $e .= $sep if (@l);
    $r .= $e;
  }
  return $r;
}

#
# We need to get the 8.3 name.
# We use the Win32::API module to import the function
#
sub GetShortPathName {
  my $GetShortPathName = new Win32::API('Kernel32', 'GetShortPathName', 'PPN', 'N');
  if(not defined $GetShortPathName) {
    die "Can't import API GetShortPathName: $!\n";
  }
  my $lpBuffer = " " x 260;
  my $res = $GetShortPathName->Call($_[0], $lpBuffer, 260);
  return ($res == 0 ? '' : substr($lpBuffer, 0, $res));
}

#
# Look for which component holds which file
#
sub get_component_from_file {
  my ($path) = @_;
  if (! &is_absolute($path)) {
    $path = "$opt_master_dir/$path";
  }
  my $file = $filemapping{$path};
#  print "path = $path -> file = $file -> component = " . ${$tables->{'File'}->{$file}}[1] . "\n";
  return ${$tables->{'File'}->{$file}}[1];
}

sub get_file_from_pathname {
  my ($path) = @_;
  if (! &is_absolute($path)) {
    $path = "$opt_master_dir/$path";
  }
  return $filemapping{$path};
}
  
#
# Look for which feature holds which component
#
sub get_feature_from_component {
  my ($component) = @_;
  foreach my $key (keys %{$tables->{'FeatureComponents'}}) {
    my $f = ${$tables->{'FeatureComponents'}->{$key}}[0];
    if (index($f,$component) > 0) {
      return $key;
    }
  }
  print STDERR "Couldn't find the feature for component $component\n";
  return '';
}

#
# Get the parent feature
#
sub get_feature_parent {
  my ($f) = @_;
  my $count = $#{$required_by{$f}};
  my $parent = ${$required_by{$f}}[0];
  $parent =~ s@^.*/([^/]*)$@\1@;
  $parent =~ s/[_-]//g;
  $parent =~ s@^Documentation@Doc@g;

  if ($count > 0) { print "$f required by " . ($count + 1) . " packages\n"; }

  print "get_feature_parent($f) = $parent\n";
  return ($count, $parent);
}

#
# Returns the directory key associated with $dir in the Directory Table
# Create it if it does not exist.
#
sub dir_key {
  my ($dir) = @_;
  my @path = split ("/", "$dir");
  my $shortdir = &GetShortPathName("$opt_master_dir/$dir");
  $shortdir =~ s@^${short_master_dir}/@@;
#  print "short_master_dir = $short_master_dir\nshortdir = $shortdir\n";
  my @shortpath = split ("/", "$shortdir");
  my $key = "";
  my $parent = "INSTALLDIR";

  while (@path) {
    my $dir = shift @path;
    my $keydir = $dir;
    $keydir =~ s/^texmf-doc/texmfd1/;
    $keydir =~ s/^texmf-dist/texmfd2/;
    $keydir =~ s/[_-]//g;
    my $shortdir = shift @shortpath;
    $key = "$key$keydir";
    if ($tables->{'Directory'}->{$key} eq undef) {
      if ($shortdir ne $dir) {
	$dir = "$shortdir|$dir";
      }
      @{$tables->{'Directory'}->{$key}} = ($key, $parent, "$dir");
    }
    $parent = $key;
  }
  return $key;
}

################################################################################
#                                                                              #
# GUIDS management                                                             #
#                                                                              #
################################################################################

#
# There are 2 tables : one of 16000 GUIDS available for use
#                      one of for component / guid association
#

sub read_guids {
  # read the GUID
  @guids_list = ();
  open GUID, "<tl-guids.txt";
  while (<GUID>) {
    chomp $_;
    push @guids_list, $_;
  }
  close GUID;

  open GUID, "<tl-components-guids.txt";
  $guids_count = <GUID>;
  chomp $guids_count;
  while (<GUID>) {
    chomp $_;
    $_ =~ m/^\s*(.*)\s*,\s*(.*)\s*$/x;
    $guids_table{$1} = $2
  }
  close GUID;
}

sub save_guids {
  unlink("tl-components-guids.txt.bak");
  &move("tl-components-guids.txt", "tl-components-guids.txt.bak");
  open GUID, ">tl-components-guids.txt";
  print GUID $guids_count . "\n";
  foreach my $k (keys %guids_table) {
    print GUID $k . "," . $guids_table{$k} . "\n";
  }
  close GUID;
  unlink("tl-components-guids.txt.bak");
}

sub get_guid {
  my $pattern = $_;
  my $guid = $guids_table{$pattern};

  if ($guid eq undef or $guid eq '') {
    $guid = $guids_list[$guids_count++];
    $guids_table{$pattern} = $guid;
    print "new: pattern = $pattern, guid = $guid\n";
  }
  return $guid;
}

################################################################################
#                                                                              #
# TABLES management                                                            #
#                                                                              #
################################################################################

sub do_patterns {
  my ($feature, $condition, @patterns) = @_;
  # Iterate over all patterns
  map {
    my $pattern = $_;
    my $component = "$feature$component_count";

    if ($tables->{'Component'}->{$component}) {
      print STDERR "Error: component $component already exists !\n";
    }
    my $directory = &dir_key(&FileUtils::dirname($pattern));
    my $component_id = &get_guid($pattern);
    my $keypath = '';

    # delay creation of the component because we need a keypath
    my @files = &FileUtils::globexpand(0, "$opt_master_dir/$pattern");

    map {
      my $file = "file$file_sequence"; # &FileUtils::basename($_);
      $keypath = $file if (! $keypath);
      # Memorize the list of files per feature for &do_media() later on
      ${${cab}->{$feature}->{$_}} = $file;
      #      print "test " ; map { print "$_ "; } (keys %{${cab}}); print " \n";
      my $fullname = $_;
      my $filename = &FileUtils::basename($fullname);
      my $shortfile = &FileUtils::basename(&GetShortPathName($fullname));
      my @st = stat($fullname);
      my $filesize = $st[7];
      my $version = '';
      my $language = '';
      if ($filename =~ m/\.(exe|dll)$/) {
	($version, $language) = split ("\n", `getversion "$filename"`);
	$version =~ s/,/./g;
      }
      
      my $attributes = ($filename =~ /(\.exe|\.dll)$/ ? 1024 : 0);
      $filename = "$shortfile|$filename" if ($filename ne $shortfile);
      @{$tables->{'File'}->{$file}} = ($file, $component, $filename, $filesize, $version, $language, $attributes, $file_sequence);
      $filemapping{$fullname} = $file;
      $file_sequence++;
    } @files;
    
    if ($#files == -1) {
      my $dir = &FileUtils::dirname($pattern);
      $parent = &FileUtils::dirname($dir);
      $dir = &FileUtils::basename($dir);
      # Empty pattern, need to list the Directory/Component pair in the CreateFolders table
      @{$tables->{'CreateFolder'}->{$directory}} = ($directory, $component);
      # Add it to the Directory table too
      @{$tables->{'Directory'}->{$directory}} = ($directory, $parent, "$dir");

    }
    
    @{$tables->{'Component'}->{$component}} = ($component, $component_id, $directory, 0, $condition, $keypath);
    $component_count++;

  } @patterns;
}

my @level_recommended = ();
my @level_full = ();

sub get_level {
  my ($tpmname) = @_;
  my $level = 500;
  if (! @level_recommended) {
    my $tpm = Tpm->new("TLCore/scheme-recommended.tpm");
    @level_recommended = $tpm->getRequiredTpm(1);
    print "Got required tpms for scheme-recommended\n";
  }
  if (! @level_full) {
    my $tpm = Tpm->new("TLCore/scheme-full.tpm");
    @level_full = $tpm->getRequiredTpm(1);
    print "Got required tpms for scheme-full\n";
  }
  if (&FileUtils::member($tpmname, @level_recommended)) {
    $level = 100;
  }
  print "level($tpmname) = $level\n";
  return $level;
}

my $feature_display = 2;

sub do_tpm {
  my ($tpmname) = @_;
  my $tpm = Tpm->new($tpmname);
  my $feature = $tpm->getAttribute('Provides');
  $feature =~ s@[/-]@@g;
  $feature =~ s@^Documentation@Doc@g;
  
  # Add the feature and the components
  my $feature_parent;

  my $title = $tpm->getAttribute('Name');

  my $description = $tpm->getAttribute('Title');
  $description =~ s/\n//gxo;
  if (length($description) > 64) {
    $description =~ s/\. .*$//;
  }
  if (length($description) > 64) {
    $description =~ s/[ \t][^ \t]*$//;
  }
  if ($description eq '' || length($description) > 64) {
    print STDERR "Cutting title for $feature from $description to ";
    $description = $tpm->getAttribute('Name');
    print STDERR "$description\n";
  }

#   my $description = substr ($tpm->getAttribute('Description'), 0, 255);
#   $description =~ s/\n//gxo;
#   $description =~ s/\t/ /gxo;

  my $level = 1;
  if ($product =~ m/TeXLive/) {
    &get_level($tpmname);		# see InstallLevel
  }
  my $directory = '';
  my $attributes = 0;
  my $display;

  # If it is a collection
  if ($tpm->getAttribute('Name') =~ /collection-/) {
    $feature_parent = '';
    $attributes = 0;
  }
  # or if it is a package or tlcore
  else {
    $feature_parent = "&get_feature_parent(\"$feature\")";
    $attributes = 18;
  }

  $display = $feature_display;
  $feature_display += 2;

  my %req = $tpm->getHash('Requires');
  foreach my $subdir (keys %req) {
    map { 
      $_ = "$subdir$_";
      $_ =~ s@[_-]@@g;
      $_ =~ s@^Documentation@Doc@g;
      push @{$required_by{$_}}, $feature ;
      print "$_ is requirerd by @{$required_by{$_}}\n";
    } @{$req{$subdir}}
  }
  if ($tables->{'Feature'}->{$feature}) {
    print STDERR "Error: feature $feature already exists!\n";
  }
  @{$tables->{'Feature'}->{$feature}} = ($feature, $feature_parent, $title, $description, $display, $level, $directory, $attributes);

  # Create one component per line of patterns
  # Mandatory with MSI: any component must consist of files in the same directory
  # Regular components
#   if ($tpm->getAttribute('Type') eq 'Package') {
#     $tpm->patternsUpdate();
#   }
#   elsif ($tpm->getAttribute('Type') eq 'Documentation') {
#     $tpm->patternsUpdate();
#   }
  $tpm->patternsUpdate();

  my $start_component = $component_count;
  my @patterns = $tpm->getList("BinPatterns");
  @patterns = grep { $_ =~ m/win32/ } @patterns;
  @patterns = grep { $_ !~ m/win32-static/ } @patterns;
  push @patterns, $tpm->getList("RunPatterns");
  &do_patterns($feature, '', @patterns); 
  # Doc components
  @patterns = $tpm->getList("DocPatterns");
#  &do_patterns($feature, 'DOC', @patterns); 
  &do_patterns($feature, '', @patterns); 
  # Source components
  @patterns = $tpm->getList("SourcePatterns");
#  &do_patterns($feature, 'SOURCE', @patterns); 
  &do_patterns($feature, '', @patterns); 

  # List components in the feature
  my $feature_components = "";
  for (my $i = $start_component; $i < $component_count; $i++) {
    # we put the raw line here because there will be multiple entries with the same key
    $feature_components .= "$feature\t$feature$i\n";
  }
  print "$feature = $feature_components\n";
  @{$tables->{'FeatureComponents'}->{$feature}} = ($feature_components);
  # Add files

}

sub do_shortcuts {
  # Add shortcuts
  print "Doing Shortcuts...\n" if ($opt_verbose);
  
#  @{$tables->{'Registry'}->{"registryshortcuts"}} = ("registryshortcuts", -1, "Software\\TeX Users Group\\${product}", 'Shortcuts', '1', "componentshortcuts");
  @{$tables->{'Component'}->{'componentshortcuts'}} = ('componentshortcuts', &get_guid("componentshortcuts"), 'ProductMenuFolder', 0, '', '');
  if ($product eq 'TeXLive') {
    @{$tables->{'FeatureComponents'}->{$feature}} = ('TLCoreschemefull', 'componentshortcuts');
  }
  elsif ($product eq 'XEmTeX') {
    @{$tables->{'FeatureComponents'}->{$feature}} = ('TLCoreschemexemtex', 'componentshortcuts');
  }
  elsif ($product eq 'XEmacs') {
    @{$tables->{'FeatureComponents'}->{$feature}} = ('TLCoreschemexemacs', 'componentshortcuts');
  }
  
  map {
    if ($_ eq 'windvi') {
      # Windvi
      my $component = &get_component_from_file('bin/win32/windvi.exe');
      my $feature = &get_feature_from_component($component);
      
      @{$tables->{'Shortcut'}->{'windvi'}} = ('windvi.exe', 'ProductMenuFolder', 'windvi.exe', $component, $feature, '-single', 'DVI file viewer', 0, 'windvi.ico', 1, 1, '');
    }
    elsif ($_ eq 'url') {
      my $component = 'componentshortcuts';
      my $feature = 'TLCoreschemexemtex';
      if ($product eq 'TeXLive') {
	@{$tables->{'Shortcut'}->{'fpTeXHome'}} = ('fpTeXHome', 'ProductMenuFolder', 'FPTEX\'~1|fpTeX\'s Home', $component, '[FPTEXHOME]', '', '', 0, '', 0, 1, '');
	@{$tables->{'Shortcut'}->{'TeXLiveHTML'}} = ('TeXLiveHTML', 'ProductMenuFolder', 'TEXLIV~1|TeXLive Documentation', $component, "[#" . &get_file_from_pathname("texmf-doc/doc/english/texlive-en/live.html") . "]", '', '', 0, '', 0, 1, '');
      }
      elsif ($product eq 'XEmTeX') {
	@{$tables->{'Shortcut'}->{'XEmTeXHome'}} = ('XEmTeXHome', 'ProductMenuFolder', 'XEMTEX~1|XEmTeX\'s Home', $component, '[XEMTEXHOME]', '', '', 0, '', 0, 1, '');
      }
      @{$tables->{'Shortcut'}->{'GUTHome'}} = ('GUTHome', 'ProductMenuFolder', 'GUTenb~1|GUTenberg\'s Home', $component, '[GUTENBERG]', '', '', 0, '', 0, 1, '');
      @{$tables->{'Shortcut'}->{'Uninstall'}} = ('Uninstall', 'ProductMenuFolder', "Remove|Remove ${product}", $component, '[SystemFolder][MsiexecProp]', '/x [ProductCode]', '', 0, '', 0, 1, '');
    }
    elsif ($_ eq 'winclient') {
      # Winclient
      my $winclient_component = '';
      my $target = '';
      if ($product eq 'XEmTeX' || $product eq 'TeXLive') {
	$winclient_component = &get_component_from_file('bin/win32/xemacs.exe');
        $target = &get_file_from_pathname("bin/win32/xemacs.exe");
      }
      elsif ($product eq 'XEmacs') {
	$winclient_component = &get_component_from_file('xemacs-21.5/i586-pc-win32/xemacs.exe');
        $target = &get_file_from_pathname("xemacs-21.5/i586-pc-win32/xemacs.exe");
      }
      my $winclient_feature = &get_feature_from_component($winclient_component);
      my $winclient_icon = ($product =~ m/^xemtex/i ? 'xemtex.ico' : 'xemacs.ico');
      my $winclient_icon_index = ($product =~ m/^xemtex/i ? 1 : 0);
      # KeyPath should be taken into HKCU
#      @{$tables->{'Component'}->{$winclient_component}}[3] = 4; 
#      @{$tables->{'Component'}->{$winclient_component}}[5] = 'WinclientShortcut'; 

#      @{$tables->{'Registry'}->{'WinclientShortcut'}} = ('WinclientShortcut', -1, 'Software\\XEmacs Team\\XEmacs', 'XEmacs', "[INSTALLDIR]\\XEmacs\\XEmacs-21.5\\i586-pc-win32\\xemacs.exe", $winclient_component);

      @{$tables->{'Shortcut'}->{'Winclient1'}} = ('Winclient1', 'ProductMenuFolder', 'xemacs.exe', $winclient_component, ${winclient_feature}, '', "${product} Editor", 0, $winclient_icon, $winclient_icon_index, 1, '');	
      @{$tables->{'Shortcut'}->{'Winclient2'}} = ('Winclient2', 'DesktopFolder', 'xemacs.exe', $winclient_component, ${winclient_feature}, '', "${product} Editor", 0, $winclient_icon, $winclient_icon_index, 1, '');	
    }
  } @_;
}

sub do_extensions {
  # Add extensions
  print "Doing Extensions...\n" if ($opt_verbose);
  my %prog = ('dvi' => 'bin/win32/windvi.exe',
	      'tex' => 'bin/win32/winclient.exe',
	      'bib' => 'bin/win32/winclient.exe');
  map {
    my $component = &get_component_from_file($prog{$_});
    my $feature = &get_feature_from_component($component);
    @{$tables->{'Extension'}->{$_}} = ($_, $component, "$_.Document", '', $feature);
  } @_;
}

sub do_progids {
  # Add ProgIds
  my $winclient_icon = ($product =~ m/^xemtex$/i ? 'xemtex.ico' : 'xemacs.ico');
  my %icons = ('dvi' => 'windvi.ico',
	       'tex' => $winclient_icon,
	       'bib' => $winclient_icon);
  my %labels = ('dvi' => 'DVI document',
		'tex' => 'TeX document',
		'bib' => 'BibTeX document');
  print "Doing ProgIds...\n" if ($opt_verbose);
  map {
    @{$tables->{'ProgId'}->{"$_.Document"}} = ("$_.Document", '', '', $labels{$_}, $icons{$_}, 1);
  } @_;
}

sub do_registry {
  # Add Registry
  my $registry_count = 0;
  print "Doing Registry...\n" if ($opt_verbose);
  map {
    if (m/ghostscript/i) {
      # Ghostscript
      # Add component to the component table
      @{$tables->{'Component'}->{'registryTLCorebinghostscript'}} = ('registryTLCorebinghostscript', &get_guid('registryTLCorebinghostscript'), 'TARGETDIR', 4, '', "registry${registry_count}");
      my $fc;
      ($fc) = @{$tables->{'FeatureComponents'}->{'TLCorebinghostscript'}};
      $fc .= "TLCorebinghostscript\tregistryTLCorebinghostscript\n";
      @{$tables->{'FeatureComponents'}->{'TLCorebinghostscript'}} = ($fc);
      print "registry ghostscript\n";
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\GNU Ghostscript\\7.07', 'GS_DLL', "[INSTALLDIR]bin\\win32\\gsdll32.dll", "registryTLCorebinghostscript");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\GNU Ghostscript\\7.07', 'GS_LIB', "[INSTALLDIR]xemtex\\gslib;[INSTALLDIR]xemtex\\gsfonts", "registryTLCorebinghostscript");
      $registry_count++;
    }
    elsif (m/imagemagick/i) {
      # ImageMagick
      # Add component to the component table
      @{$tables->{'Component'}->{'registryTLCorebinimagemagick'}} = ('registryTLCorebinimagemagick', &get_guid('registryTLCorebinimagemagick'), 'TARGETDIR', 4, '', "registry${registry_count}");
      my $fc;
      ($fc) = @{$tables->{'FeatureComponents'}->{'TLCorebinimagemagick'}};
      $fc .= "TLCorebinimagemagick\tregistryTLCorebinimagemagick\n";
      @{$tables->{'FeatureComponents'}->{'TLCorebinimagemagick'}} = ($fc);
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\5.5.7\\Q:16', 'BinPath', '[INSTALLDIR]bin\\win32', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\5.5.7\\Q:16', 'ConfigurePath', '[INSTALLDIR]bin\\win32\\config', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\5.5.7\\Q:16', 'LibPath', '[INSTALLDIR]bin\\win32', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\5.5.7\\Q:16', 'CoderModulesPath', '[INSTALLDIR]bin\\win32\\modules\\coders', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\5.5.7\\Q:16', 'FilterModulesPath', '[INSTALLDIR]bin\\win32\\modules\\filters', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\Current', 'Version', 'MagickPackageVersion', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\Current', 'QuantumDepth', 16, "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\Current', 'BinPath', '[INSTALLDIR]bin\\win32', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\Current', 'ConfigurePath', '[INSTALLDIR]bin\\win32\\config', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\Current', 'LibPath', '[INSTALLDIR]bin\\win32', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\Current', 'CoderModulesPath', '[INSTALLDIR]bin\\win32\\modules\\coders', "registryTLCorebinimagemagick");
      $registry_count++;
      @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", -1, 'SOFTWARE\\ImageMagick\\Current', 'FilterModulesPath', '[INSTALLDIR]bin\\win32\\modules\\filters', "registryTLCorebinimagemagick");
    }
#     elsif (m/shortcuts/i) {
#       $registry_count++;
#       @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", 1, 'Software\\TeX Users Group\\TeXLive', 'FpTeXWebPath', 'http://www.fptex.org/', "shortcuts");
#       $registry_count++;
#       @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", 1, 'Software\\TeX Users Group\\TeXLive', 'GutenbergWebPath', 'http://www.gutenberg.eu.org/', "shortcuts");
#       $registry_count++;
#       @{$tables->{'Registry'}->{"registry${registry_count}"}} = ("registry${registry_count}", 1, 'Software\\TeX Users Group\\TeXLive', 'TeXLiveDocPath', '[TARGETDIR]TeXLive\\texmf\doc\tldoc\english\live.html', "componentshortcuts");
#     }
    else {
      print "Registry: unknown registry item $_\n";
    }
  } @_;
}

sub do_duplicatefiles {
  # Add Duplicate Files

}

sub do_environment {
  # Add Environment Variables
  print "Doing Environment...\n" if ($opt_verbose);
  if (${product} eq 'XEmacs') {
#    my $component = &get_component_from_file("xemacs-21.5/i586-pc-win32/xemacs.exe");
#    @{$tables->{'Environment'}->{'PATH'}} = ('PATH', '=-PATH', "[~];[SHORTINSTALLDIR]xemacs-21.5\\bin\\i586-pc-win32", $component);
  }
  else {
    my $component = &get_component_from_file("bin/win32/tl90kpse.dll");
    @{$tables->{'Environment'}->{'PATH'}} = ('PATH', '=-PATH', "[~];[SHORTINSTALLDIR]bin\\win32", $component);
    if (! -d "${opt_master_dir}/texmf-local") { mkdir ("${opt_master_dir}/texmf-local", 777); }
    if (! -d "${opt_master_dir}/temp") { mkdir ("${opt_master_dir}/temp", 777); }
    my $dirkey = &dir_key("texmf-local");
    @{$tables->{'CreateFolder'}->{$dirkey}} = ($dirkey, $component);
    $dirkey = &dir_key("texmf-var/web2c");
    @{$tables->{'CreateFolder'}->{$dirkey}} = ($dirkey, $component);
    #  &dir_key("texmf-var/tex4ht/base");
    #  &dir_key("texmf-var/dvipdfm/config");
    #  &dir_key("texmf-var/pdftex/config");
    #  &dir_key("texmf-var/tex/generic/config");
    # No need to set this one anymore 07/2004
    #  @{$tables->{'Environment'}->{'TEXMFCNF'}} = ('TEXMFCNF', '=-TEXMFCNF', "[TARGETDIR]${product}\\texmf-var\\web2c", $component);
    @{$tables->{'Environment'}->{'TEXMFCNF'}} = ('TEXMFCNF', '!-TEXMFCNF', "", $component);
    $dirkey = &dir_key("temp");
    @{$tables->{'CreateFolder'}->{$dirkey}} = ($dirkey, $component);
    @{$tables->{'Environment'}->{'TEXMFTEMP'}} = ('TEXMFTEMP','=-TEXMFTEMP', "[SHORTINSTALLDIR]temp", $component);
    @{$tables->{'Environment'}->{'PERL5LIB'}} = ('PERL5LIB', '=-PERL5LIB', "[SHORTINSTALLDIR]xemtex\\perl\\site\\lib;[SHORTINSTALLDIR]xemtex\\perl\\lib", &get_component_from_file("bin/win32/perl58.dll"));
  }
}

sub do_customaction {
  # Add Custom Actions
  # Build language.dat and updmap.cfg
  my ($name, $type, $dll, $function, $sequence, $message) = @_;

  @{$tables->{'Binary'}->{$dll}} = ($dll, "$dll.ibd");

  @{$tables->{'CustomAction'}->{$name}} = ($name, $type, $dll, $function);
  @{$tables->{'InstallExecuteSequence'}->{$name}} = ($name, '', $sequence);
  @{$tables->{'AdminExecuteSequence'}->{$name}} = ($name, '', $sequence);
  @{$tables->{'ActionText'}->{$name}} = ($name, $message, '');

}

sub do_copy_or_process {
  my ($src, $dst) = @_;
  # If we change the name, do not forget to change the entry in the 'File' table
  if ($src =~ m@/texmf-dist/doc/@) {
    if ($src =~ m@\.ps\.gz@) {
    }
#    elsif ($src =~ m@\.ps@) {
#    }
    elsif ($src =~ m@\.dvi@) {

    }
  }
  &FileUtils::copy($src, $dst);
}

sub do_media {
  # Add Media
  print "Doing Media and Cabs...\n" if ($opt_verbose);
  my $name = $_;
  my $cabscount = 1;
  my $firstfile = 1;
  my $lastfile = 1;
  my $modified = 0;
  my @cabfiles = ();

  if ($product eq 'TeXLive') {
    # $file_sequence is the number of the last file
    @{$tables->{'Media'}->{$cabscount}} = ($cabscount, $file_sequence, "${product}", '', "${product}", '');
  }
  elsif ($product eq 'XEmTeX' || $product eq 'XEmacs') {
    # Building one .cab per feature
    foreach my $feature (keys %{${cab}}) {
      $lastfile = 1;
      print "feature = $feature\n" if ($opt_verbose);
      if (! -d "${tmpdir}/files/$feature") {
	mkdir "${tmpdir}/files/$feature" ;
      }
      @cabfiles = ();
      open OUT, ">copy.log";
      foreach my $pathname (keys %{${cab}->{$feature}}) {
	my $filename = ${${cab}->{$feature}->{$pathname}};
	print OUT "$filename <- $pathname";
	# Copy the file to the temp area if not already there or changed
	
	if ($opt_rebuild_cab 
	    && ((! -f "${tmpdir}/files/$feature/$filename") 
		||  (&FileUtils::newer("$pathname", "${tmpdir}/files/$feature/$filename") > 0))) {
	  &do_copy_or_process("$pathname", "${tmpdir}/files/$feature/$filename");
	  print OUT " ... copied";
	  push @cabfiles, $filename;
	  $modified = 1;
	}
	print OUT "\n";
	my $l = $filename;
	$l =~ s/^file//;
	$lastfile = ($l > $lastfile ? $l : $lastfile);
      }
      my $working_dir = &getcwd();
      # Cab the list of files
      # Beware ... we need to make sure the sequence order in the cab file is the same
      # than the one stated in the File table
      # Roughly, we need to sort the file names numerically
      if ($opt_rebuild_cab && ($modified || ! -f "${tmpdir}\\Cabs\\$feature.cab.ibd")) {
	@cabfiles = sort { substr($a, 4) <=> substr($b, 4);} @cabfiles;
	chdir("${tmpdir}\\files\\$feature");
	#      print "cabarc n ..\\..\\Cabs\\$feature.cab.ibd " .  (join " ", @cabfiles) . "\n" if ($opt_verbose);
	open FOO, ">${tmpdir}\\cabs\\${feature}.lst";
	print FOO (join "\n", @cabfiles);
	close FOO;
	system("cabarc n ${tmpdir}\\Cabs\\${feature}.cab.ibd \@${tmpdir}\\Cabs\\${feature}.lst");
	# It seems that msidb imports both the Cabs table and the binary streams.
	chdir("${working_dir}");
	#      chdir("Cabs");
	#      system("msidb -d c:\\Source\\TeXLive\\$opt_target.msi -k Cabs.$feature.cab");
	#      system("msidb -d c:\\Source\\TeXLive\\$opt_target.msi -a $feature.cab.ibd");
	#      chdir("..");
	# Do not create this table if we are not ready to rebuild the cab files
	@{$tables->{'Cabs'}->{$feature}} = ($feature . ".cab", $feature . ".cab.ibd", );
      }
      $modified = 0;
      # Should we rebuild the Media table if we are not rebuilding the .cab files ?
      @{$tables->{'Media'}->{$cabscount}} = ($cabscount, $lastfile, "${product}", "#Cabs.$feature.cab", "${product}", '');
      $firstfile = $lastfile + 1;
      $cabscount++;
    }
    my @media_lines =  sort { return ${$a}[1] <=> ${$b}[1]; } (values %{$tables->{'Media'}});
    my $media_count = 1;
    map {
      my $media = $_;
      $tables->{'Media'}->{$media_count} = $_;
      ${$tables->{'Media'}->{$media_count}}[0] = $media_count;
      $media_count++;
    } @media_lines;
  }
}

sub do_icons {
  # Add Icon
  print "Doing Icons...\n" if ($opt_verbose);
  my $icon_count = 1;
  map {
    @{$tables->{'Icon'}->{$icon_count++}} = ("$_.ico", "XEmTeXIcons.exe.ibd");
  } @_;
}

sub do_directories {
  @{$tables->{'Directory'}->{'INSTALLDIR'}} = ('INSTALLDIR', 'ProgramFilesFolder', "$product:.");
  @{$tables->{'Directory'}->{'ProductMenuFolder'}} = ('ProductMenuFolder', 'ProgramMenuFolder', "$product");
}

sub do_feature {
  my ($tpmname) = @_;
  print "Doing features $tpmname\n" if ($opt_verbose);
  &do_tpm($tpmname);
  my $xemtex = Tpm->new($tpmname);
  my @requiredTpm = $xemtex->getRequiredTpm(1);
  map { print "required $_\n"; } @requiredTpm;
  map { &do_tpm($_); } @requiredTpm;
  # my %xemreq = $xemtex->getHash("Requires");
  # if (%xemreq) {
  #   foreach my $k (@{Tpm::TpmCategories}) {
  #     my @taglist = @{$xemreq{$k}};
  #     for my $tag (@taglist) {
  # 	&do_tpm("$k/$tag");
  #     }
  #   }
  # }

  # Fix the FeatureParent field. Make only collections and their
  # direct childs appear. Other features disappear.
  # Add the Condition table.
  foreach my $k (keys %{$tables->{'Feature'}}) {
    print "k = $k\n";
    my ($count, $feature_parent) = eval (${$tables->{'Feature'}->{$k}}[1]);
    ${$tables->{'Feature'}->{$k}}[1] = $feature_parent;
    if (${$tables->{'Feature'}->{$k}}[1] eq '') {
      ${$tables->{'Feature'}->{$k}}[7] = 0;
    }
    if ($count > 0) {
      # we need to make it selected and not deselectable
      # This is $level
      ${$tables->{'Feature'}->{$k}}[5] = 1;
      # This is $display
      ${$tables->{'Feature'}->{$k}}[4] = 0;
      ${$tables->{'Feature'}->{$k}}[7] = 0;
    }

  }
  #  &FileUtils::walk_tree($file_tree, do_tables);

}

sub do_condition {
  # Condition table
  print "Doing Condition...\n" if ($opt_verbose);
  @{$tables->{'Condition'}} = { };
  #  foreach my $key (keys %{$tables->{'Feature'}}) {
  #    if (@{$required_by{$key}}) {
  #      my $condition = join(" OR ", map { $_ = "&" . $_ . "=3"} @{$required_by{$key}});
  #      print "$key\t3\t$condition\n";
  #      @{$tables->{'Condition'}->{$key}} = ($key, 3, $condition);
  #    }
  #  }
}

sub do_property {
  my $property_count = 0;
  @{$tables->{'Property'}->{$property_count++}} = ('_WiseDebugMode', '0');
  @{$tables->{'Property'}->{$property_count++}} = ('_WiseDialogFontDefault', '{&MSSansSerif8}');
  @{$tables->{'Property'}->{$property_count++}} = ('_WiseDialogSuffix', 'Setup');
  @{$tables->{'Property'}->{$property_count++}} = ('_WiseDialogTitleFontDefault', '{&Arial8}');
  @{$tables->{'Property'}->{$property_count++}} = ('Accept', 'No');
  @{$tables->{'Property'}->{$property_count++}} = ('ApplicationUsers', 'AllUsers');
  @{$tables->{'Property'}->{$property_count++}} = ('DefaultUIFont', 'Arial10');
  @{$tables->{'Property'}->{$property_count++}} = ('DiskPrompt', '[ProductName] [1]');
  @{$tables->{'Property'}->{$property_count++}} = ('ErrorDialog', 'ErrorDialog');
  @{$tables->{'Property'}->{$property_count++}} = ('GUTENBERG', 'http://www.gutenberg.eu.org/');
  if ($product eq 'TeXLive') {
    @{$tables->{'Property'}->{$property_count++}} = ('FPTEXHOME', 'http://www.fptex.org/');
    @{$tables->{'Property'}->{$property_count++}} = ('TEXLIVEHTML', 'TARGETDIR\TeXLive\texmf-doc\doc\english\texlive-en\live.html');
    @{$tables->{'Property'}->{$property_count++}} = ('XEMACS', 'TARGETDIR\bin\win32\xemacs.exe');
  }
  elsif ($product eq 'XEmTeX') {
    @{$tables->{'Property'}->{$property_count++}} = ('XEMTEXHOME', 'http://www.fptex.org/xemtex/');
    @{$tables->{'Property'}->{$property_count++}} = ('XEMACS', 'TARGETDIR\bin\win32\xemacs.exe');
  }
  elsif ($product eq 'XEmacs') {
    @{$tables->{'Property'}->{$property_count++}} = ('XEMACS', 'TARGETDIR\xemacs-21.5\i586-pc-win32\xemacs.exe');
  } 	
  if (${product} eq 'XEmacs') {
  }
  else {
    # Do we need that ?
#    @{$tables->{'Property'}->{$property_count++}} = ('INSTALLDIR', 'INSTALLDIR');
    @{$tables->{'Property'}->{$property_count++}} = ('INSTALLLEVEL', '100');
  }
  @{$tables->{'Property'}->{$property_count++}} = ('InstallMode', 'Typical');
  @{$tables->{'Property'}->{$property_count++}} = ('MaintenanceMode', 'Modify');
  if ($product =~ m/^texlive/i) {
    @{$tables->{'Property'}->{$property_count++}} = ('Manufacturer', 'TeX Users Group');
  }
  elsif ($product =~ m/^xemtex/i) {
    @{$tables->{'Property'}->{$property_count++}} = ('Manufacturer', 'Suplec - EDP Sciences');
  }
  elsif ($product =~ m/^xemacs/i) {
    @{$tables->{'Property'}->{$property_count++}} = ('Manufacturer', 'Suplec');
  }
  @{$tables->{'Property'}->{$property_count++}} = ('MsiexecProp', 'msiexec');
  @{$tables->{'Property'}->{$property_count++}} = ('PALMUSERS', '0');
  @{$tables->{'Property'}->{$property_count++}} = ('PIDTemplate', '12345<###-%%%%%%%>@@@@@');
  @{$tables->{'Property'}->{$property_count++}} = ('ProductCode', '{53EB1169-805C-4A3A-86B1-7F170D2B30EE}');
  @{$tables->{'Property'}->{$property_count++}} = ('ProductID', 'none');
  @{$tables->{'Property'}->{$property_count++}} = ('ProductLanguage', '1033');
  @{$tables->{'Property'}->{$property_count++}} = ('ProductName', ${product});
  @{$tables->{'Property'}->{$property_count++}} = ('ProductVersion', ${product_version});
  @{$tables->{'Property'}->{$property_count++}} = ('ReinstallFileOlderVersion', 'o');
  @{$tables->{'Property'}->{$property_count++}} = ('ReinstallFileVersion', 'o');
  @{$tables->{'Property'}->{$property_count++}} = ('ReinstallRepair', 'r');
  @{$tables->{'Property'}->{$property_count++}} = ('SecureCustomProperties', 'INSTALLDIR');
  @{$tables->{'Property'}->{$property_count++}} = ('UpgradeCode', '{408F778A-3E0E-4384-BEA3-F4774D5B95BB}');
  @{$tables->{'Property'}->{$property_count++}} = ('WiseCRLF', '');
  @{$tables->{'Property'}->{$property_count++}} = ('WiseInitAdminError', 'You must have administrator rights to run this installation. Please login as an administrator and re-run this installation.');
  @{$tables->{'Property'}->{$property_count++}} = ('WiseInitExistError', '%s Version %s is already installed. You must uninstall the existing version before installing %s Version %s. Do you want to uninstall the existing version of %s?');
  @{$tables->{'Property'}->{$property_count++}} = ('WiseInitLangDefault', 'English,1033');
  @{$tables->{'Property'}->{$property_count++}} = ('WiseInitPrefix', 'Initializing');
  @{$tables->{'Property'}->{$property_count++}} = ('WiseInitSpaceError', 'Could not create temporary file, not enough free temporary disk space. Please free up disk space and rerun this installation.');
  @{$tables->{'Property'}->{$property_count++}} = ('WiseInitSuffix', 'Wizard...');
}

sub rewrite_dbs {
  # Export tables
  # $cmd = "msidb -e -d$opt_msi -fc:\source\fptex\mswin32\MSI " . join(" ", (keys %{$tables}));
  # system($cmd);
  my ($msi) = @_;

  # Fixing features
  $display = 2;
  my @kl = keys %{$tables->{'Feature'}};
  foreach my $k (
		 sort {
		   my $x0 = ${$tables->{'Feature'}->{$a}}[0];
		   my $x1 = ${$tables->{'Feature'}->{$a}}[1];
		   my $y0 = ${$tables->{'Feature'}->{$b}}[0];
		   my $y1 = ${$tables->{'Feature'}->{$b}}[1];
		   my $n = ($x1 cmp $y1);
		   if ($n == 0) {
		     $n = ($x0 cmp $y0);
		   }
		   return $n;
		 }
		 @kl) {
    if (${$tables->{'Feature'}->{$k}}[4] != 0) {
      ${$tables->{'Feature'}->{$k}}[4] = $display;
      $display += 2;
    }
  }
    
  print "Rewriting tables\n" if ($opt_verbose);

  # Rewrite tables
  foreach my $t (keys %{$tables}) {
    # copy template into current directory
    my $ft = substr($t, 0, 8) . ".idt";
    &FileUtils::copy("Template/$t.idt", "${tmpdir}/$ft");
    # add the lines
    open IDT, ">>${tmpdir}/$ft";
    binmode(IDT);
    print "Table $t:\n";
    foreach my $k (sort { if ($a =~ /^[0-9]*$/) {
                            return $a <=> $b;
			  }
			  else {
			    return $a cmp $b;
			  }
			}
		   (keys %{$tables->{$t}})) {
      my $line = &join_raw("\t",  @{$tables->{$t}->{$k}});
      $line =~ s/\n*$/\n/x;
      if ($line ne '') {
	print IDT $line;
      }
    }
    close IDT
  }

  # Import tables
  foreach my $table (keys %{$tables}) {
    my $status;
    my $cmd = "msidb -i -dc:\\Source\\TeXLive\\$msi -f${tmpdir} " . substr($table, 0, 8);
    print "$cmd\n";
    system($cmd);
    do {
      $status = `tasklist /fi "imagename eq msidb" 2>nul`;
      print "status = $status\n";
    } until ($status eq '');
    
  }
  
  my $logfile = "validate-$msi";
  $logfile =~ s/\.msi$/.log/;
  my $cmd = "c:/progra~1/msival2/msival2 c:\\Source\\TeXLive\\$msi c:/progra~1/msival2/xplogo.cub -l ${opt_logdir}/$logfile";
  system($cmd);
}

sub do_texlive {
  print "Doing TeXLive.\n" if ($opt_verbose);
  $product = 'TeXLive';
  $product_version = $texlive_version;
  &do_directories();
  &do_feature("TLCore/scheme-full");
  &do_condition();
  &do_shortcuts('winclient', 'windvi', 'url');
  &do_extensions('tex', 'bib', 'dvi');
  &do_progids('tex', 'bib', 'dvi');
  &do_registry('ghostscript', 'imagemagick');
  &do_icons('xemacs', 'windvi');
  &do_duplicatefiles();
  &do_environment();
  &do_customaction('Finalize', 1, 'FinalizeDll', '_Finalize@4', 7000, 'Generating TeX support files...', '');
  &do_customaction('FixLongToShortPathName', 1, 'FinalizeDll', '_FixLongToShortPathName@4', 5150, 'Calculating Short PathName...', '');
#  &do_customaction('InstallCompressedXEmTeX', 1, 'InstallCompressedDll', '_InstallCompressedXEmTeX@4', 3900, 'Install TeX compressed files...', '');
  &do_media();
  &save_guids();
  &rewrite_dbs("texlive-uncompressed-${texlive_version}.msi");
}

sub do_texlive_inst {
  print "Doing TeXLive Installable.\n" if ($opt_verbose);
  $product = 'TeXLive';
  $product_version = $texlive_version;
  &do_directories();
  &do_feature("TLCore/scheme-full");
  &do_condition();
  &do_shortcuts('winclient', 'windvi', 'url');
  &do_extensions('tex', 'bib', 'dvi');
  &do_progids('tex', 'bib', 'dvi');
  &do_registry('ghostscript', 'imagemagick');
  &do_icons('xemacs', 'windvi');
  &do_duplicatefiles();
  &do_environment();
  &do_customaction('Finalize', 1, 'FinalizeDll', '_Finalize@4', 7000, 'Generating TeX support files...', '');
  &do_customaction('FixLongToShortPathName', 1, 'FinalizeDll', '_FixLongToShortPathName@4', 5150, 'Calculating Short PathName...', '');
  &do_media();
  &save_guids();
  &rewrite_dbs("texlive-compressed-${texlive_version}.msi");
}

sub do_texlive_demo {
  print "Doing TeXLive Demo.\n" if ($opt_verbose);
  $product = 'TeXLive_Compressed';
  $product_version = $texlive_version;
  &do_directories();
  &do_feature("TLCore/scheme-demo");
  &do_condition();
  &do_shortcuts('winclient', 'windvi', 'url');
  &do_extensions('tex', 'bib', 'dvi');
  &do_progids('tex', 'bib', 'dvi');
  &do_registry('ghostscript', 'imagemagick');
  &do_icons('xemacs', 'windvi');
  &do_duplicatefiles();
  &do_environment();
  &do_customaction('Finalize', 1, 'FinalizeDll', '_Finalize@4', 7000, 'Generating TeX support files...', '');
  &do_customaction('FixLongToShortPathName', 1, 'FinalizeDll', '_FixLongToShortPathName@4', 5150, 'Calculating Short PathName...', '');
  &do_media();
}

sub do_xemtex {
  print "Doing XEmTeX.\n" if ($opt_verbose);
  $product = 'XEmTeX';
  $product_version = $xemtex_version;
  &do_directories();
  &do_feature("TLCore/scheme-xemtex");
  &do_condition();
  &do_shortcuts('winclient', 'url');
  &do_extensions('tex', 'bib');
  &do_progids('tex', 'bib');
  &do_registry('ghostscript', 'imagemagick');
  &do_icons('xemtex');
  &do_duplicatefiles();
  &do_environment();
  &do_property();
  &do_customaction('Finalize', 1, 'FinalizeDll', '_Finalize@4', 7000, 'Generating TeX support files...', '');
  &do_customaction('FixLongToShortPathName', 1, 'FinalizeDll', '_FixLongToShortPathName@4', 5150, 'Calculating Short PathName...', '');
  &do_media();
  &save_guids();
  &rewrite_dbs("xemtex-${xemtex_version}.msi");
}

sub do_xemtex_net {
  print "Doing XEmTeX Network.\n" if ($opt_verbose);
  $product = 'XEmTeX_Net';
  $product_version = $xemtex_version;
  &do_directories();
  &do_feature("TLCore/scheme-xemtex");
  &do_condition();
  &do_shortcuts('winclient', 'url');
  &do_extensions('tex', 'bib');
  &do_progids('tex', 'bib');
  &do_registry('ghostscript', 'imagemagick');
  &do_icons('xemtex');
  &do_duplicatefiles();
  &do_environment();
  &do_customaction('Finalize', 1, 'FinalizeDll', '_Finalize@4', 7000, 'Generating TeX support files...', '');
  &do_customaction('FixLongToShortPathName', 1, 'FinalizeDll', '_FixLongToShortPathName@4', 5150, 'Calculating Short PathName...', '');
  &do_media();
  &save_guids();
  &rewrite_dbs("xemtex-net-${xemtex_version}.msi");
}

sub do_xemacs {
  print "Doing XEmacs.\n" if ($opt_verbose);
  $product = 'XEmacs';
  $product_version = $xemacs_version;
  print "tmpdir = $tmpdir\n";
  &do_directories();
  &do_feature("TLCore/scheme-xemacs");
#  &do_condition();
  &do_shortcuts('winclient');
#  &do_extensions('tex', 'bib');
#  &do_progids('tex', 'bib');
#  &do_registry('ghostscript', 'imagemagick');
  &do_icons('xemacs');
#  &do_duplicatefiles();

#  &do_environment();
#  print "Doing Environment...\n" if ($opt_verbose);
#  my $component = &get_component_from_file("xemacs-21.5/i586-pc-win32/xemacs.exe");
#  @{$tables->{'Environment'}->{'PATH'}} = ('PATH', '=-PATH', "[~];[ProgramFilesFolder]${product}\\xemacs-21.5\\i586-pc-win32", $component);
  &do_property();
#  &do_customactions();
  &do_media();
  &save_guids();
  &rewrite_dbs("xemacs-${xemacs_version}.msi");

}

sub main {

  unless (&NGetOpt ("rebuild_cab",
		    "target=s",
		    "master_dir=s",
		    "log=s",
		    "verbose",
                    "debug",
		    "help"))
    {
      print STDERR "Try `$0 --help'";
      exit 1;
    }

  $Tpm::CurrentArch = "win32";

  if ($opt_help) {
    &help;
    exit 0;
  }

  $tmpdir = "${tmpdir}-${opt_target}";

  if ($opt_master_dir) {
    die "$0: $opt_master_dir not a directory!\n" if (! -d $opt_master_dir);
    ${Tpm::MasterDir} = $opt_master_dir;
    $short_master_dir = &GetShortPathName($opt_master_dir);
  }

  &read_guids();

  if ($opt_rebuild_cab) {
    &FileUtils::rec_rmdir("${tmpdir}/Cabs");
    &FileUtils::rec_rmdir("${tmpdir}/Files");
    mkdir("${tmpdir}/Cabs");
    mkdir("${tmpdir}/Files");
  }

  print "target = $opt_target\n";
  if (${opt_target}) {
    eval("&do_" . $opt_target . "()")
  }
  else {
    print STDERR "$0: Invalid target!\n";
    exit 1;
  }

}
