################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/09/17 11:23:46 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1
USE_TEX = 1

!include <msvc/common.mak>

programs =  $(objdir)\aleph.exe 
!ifdef TEX_DLL
programs = $(programs) $(objdir)\$(library_prefix)aleph.dll
libfiles = $(objdir)\$(library_prefix)aleph.lib
!endif
manfiles = # aleph.1
#
# Object files
#
# The C sources.
aleph_c = aleph.c
aleph_o = $(objdir)\aleph.obj

objects = $(objdir)\aleph.obj $(objdir)\alephfirst.obj $(objdir)\alephbis.obj

DEFS = -I.. $(DEFS) -DTEX_ENGINE -DHAVE_CONFIG_H # -DFOO -I../foo

aleph_build = rc2

#
# Main target
#
all: $(objdir) $(programs)

!ifdef TEX_DLL

$(objdir)\$(library_prefix)aleph.exp: $(objdir)\$(library_prefix)aleph.lib

$(objdir)\$(library_prefix)aleph.lib: $(objects)
	$(archive) /DEF $(objects)

$(objdir)\$(library_prefix)aleph.dll: $(objects) $(objdir)\$(library_prefix)aleph.exp $(objdir)\aleph.res $(kpathsealib) $(texmfmplib)  $(gnuw32lib)
	$(link_dll) $(**) $(socketlibs) $(conlibs)

$(objdir)\aleph.exe: $(objdir)\main.obj $(objdir)\$(library_prefix)aleph.lib $(texmfmplib)
	$(link) $(**) $(conlibs)
!else
$(objdir)\aleph.exe: $(objects) $(objdir)\main.obj $(objdir)\aleph.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(socketlibs) $(conlibs)
!endif

aleph: $(aleph)

# C file dependencies.
aleph.c alephcoerce.h alephd.h: aleph.p $(web2c_texmf) alephextra.h
	$(web2c) aleph

# Tangling
aleph.p aleph.pool: $(otangle) aleph.web aleph.ch
	$(otangle) aleph.web aleph.ch

#
# e-Omega is build without the xml support
# so we can't reuse aleph.web and aleph.ch
# we need to build stripped down versions
#
aleph_files = \
#	omnode.ch \
	om16bit.ch \
	omstr.ch \
	omfont.ch \
	omchar.ch \
	omfi.ch \
	ompar.ch \
	omocp.ch \
	omfilter.ch \
	omtrans.ch \
	omdir.ch \
	eobase.ch \
	eofmt.ch \
	eomem.ch \
	eo16bit.ch \
	eoext.ch \
	eoeqtb.ch \
	eofix.ch \
	eocprt.ch \
	eonewdir.ch \
	eover.ch \
	eopage.ch \
	eochar.ch

aleph_changefiles = \
	com16bit.ch \
	comstr.ch \
	comfont.ch \
	comchar.ch \
	comfi.ch \
	compar.ch \
	comocp.ch \
	comfilter.ch \
	comtrans.ch \
	comdir.ch \
	comsrcspec.ch \
	ceostuff.ch

# Generation of the web and ch files.
aleph.web: $(tie) ..\tex\tex.web $(aleph_files) \
	alephextra.h aleph.defines # alephdir\aleph.mak
	$(tie) -m aleph.web ..\tex\tex.web $(aleph_files)

aleph.ch: $(tie) aleph.web $(aleph_changefiles) # alephdir/aleph.mak
	$(tie) -c aleph.ch aleph.web $(aleph_changefiles)

alephextra.h: alephextra.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

aleph.rc: aleph.rc.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

test:
	echo $(PATH)
	fmtutil --byfmt=aleph

# 
!include <msvc/config.mak>
!include <msvc/install.mak>

# Installation.
install::
	@$(copy) aleph.pool "$(texpooldir)\aleph.pool" $(redir_stdout)

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objects) aleph.c alephextra.c alephcoerce.h \
			    alephd.h aleph.p aleph.pool aleph.web aleph.ch \
			    aleph.fmt aleph.log alephextra.h aleph.rc) do \
				$(del) %%i $(redir_stderr) \
	)

!include <msvc/clean.mak>
!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
