################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/09/21 13:57:20 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1

!include <msvc/common.mak>

!ifdef STATIC

programs = 

targets =  \
	$(objdir)\ctangle.exe \
	$(objdir)\cweave.exe

!else

programs = \
	$(objdir)\ctangle.exe \
	$(objdir)\cweave.exe

targets = 

!endif

scripts =
libfiles =
includefiles =
manfiles = cweb.1
infofiles =
#
# Object files
#
objects = $(objdir)\ctangle.obj $(objdir)\cweave.obj

DEFS = -I.. $(DEFS)

#
# Main target
#
all: $(objdir) $(programs) $(targets)

# Bootstrapping ctangle requires making it with itself.  We use the opportunity
# to create an up-to-date ctangleboot as well.
$(objdir)\ctangle.exe: $(objdir)\ctangle.obj $(objdir)\cweb.obj $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(conlibs)
	$(make) ctangleboot.c 
	$(make) cwebboot.c

# ctangle.c is a special case, since it is needed to compile itself.
ctangle.c: $(objdir)\ctangleboot.exe ctangle.w ctang-w2c.ch
	@echo <<.\ctangle.bat
@echo $(verbose)
set CWEBINPUTS=.;$(srcdir)\cwebdir
.\$(objdir)\ctangleboot ctangle.w ctang-w2c.ch
set CWEBINPUTS=
<<NOKEEP
	.\ctangle.bat

cweb.c: $(objdir)\ctangleboot.exe common.w comm-w2c.ch
	.\$(objdir)\ctangleboot.exe common.w comm-w2c.ch cweb.c

$(objdir)\ctangleboot.exe: $(objdir)\ctangleboot.obj $(objdir)\cwebboot.obj $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(conlibs)

# ctangleboot.c is in the distribution
stamp-ctangle: ctangleboot.c cwebboot.c

ctangleboot.c: ctangle.w ctang-w2c.ch
	@echo <<.\ctangleboot.bat
@echo $(verbose)
echo $(make)
set CWEBINPUTS=.;$(srcdir)\cwebdir
if exist $(objdir)\ctangle.exe (
  .\$(objdir)\ctangle ctangle.w ctang-w2c.ch
  copy ctangle.c ctangleboot.c
  del ctangle.c
) else (
  touch ctangleboot.c
)
date /t >stamp-ctangle & time /t >>stamp-ctangle
set CWEBINPUTS=
$(make) $(objdir)\ctangle.exe
<<NOKEEP
	.\ctangleboot.bat

cwebboot.c: common.w comm-w2c.ch
	@echo <<.\cwebboot.bat
if exist $(objdir)\ctangle.exe (
  $(objdir)\ctangle common.w comm-w2c.ch cweb.c
  copy cweb.c cwebboot.c
  del cweb.c
) else (
  touch cwebboot.c
)
date /t >stamp-ctangle & time /t >>stamp-ctangle
$(make) $(objdir)\ctangle.exe
<<NOKEEP
	.\cwebboot.bat

$(objdir)\cweave.exe: $(objdir)\cweave.obj $(objdir)\cweb.obj $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(conlibs)

cweave.c: $(objdir)\ctangle.exe cweave.w cweav-w2c.ch
	.\$(objdir)\ctangle cweave.w cweav-w2c.ch

check: $(objdir)\cweave.exe common.w
	.\$(objdir)\cweave common.w



!include <msvc/config.mak>
!include <msvc/install.mak>

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objdir)\cweave.obj cweave.c	\
			    $(objdir)\ctangle.obj ctangle.c	\
	                    $(objdir)\cweb.obj cweb.c		\
	                    $(objdir)\ctangleboot.obj 		\
			    $(objdir)\cwebboot.obj) do $(del) %%i $(redir_stderr) \
	)
	-@$(del) common.tex common.scn common.idx $(redir_stderr)

distclean::
	-@echo $(verbose) & ( \
		for %%i in (stamp-ctangle) do $(del) %%i $(redir_stderr) \
	)

!include <msvc/clean.mak>
!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
