#include "dvipng.h"
#include <fcntl.h> // open/close
#ifndef WIN32
#include <sys/mman.h>
#include <sys/stat.h>
#endif
#if HAVE_ALLOCA_H
# include <alloca.h>
#endif

bool ReadTFM(struct font_entry * tfontp, char* tfmname)
{
#ifdef WIN32
  HANDLE filedes = INVALID_HANDLE_VALUE, mapping = 0;
  unsigned int size;
#else
  struct stat stat;
  int filedes;
#endif
  struct ft_char *tcharptr;  
  unsigned char* filemmap,*position; 
  int lh,bc,ec,nw, c;
  dviunits* width;

  DEBUG_PRINT((DEBUG_DVI|DEBUG_FT|DEBUG_TFM),
	      ("\n  OPEN METRICS:\t'%s'", tfmname));
#ifdef WIN32
  filedes = CreateFile(tfmname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if (filedes == INVALID_HANDLE_VALUE) {
    Warning("color name file %s could not be opened", tfmname);
    return(false);
  }
  mapping = CreateFileMapping(filedes, NULL, PAGE_READONLY, 0, 0, "tfmname_file");
  if (mapping == 0) {
      CloseHandle(filedes);
      filedes = INVALID_HANDLE_VALUE;
      Warning("TFM name file %s file mapping could not be created", tfmname);
      return(false);
  }
  filemmap = MapViewOfFile(mapping, FILE_MAP_READ, 0, 0, 0);
  if (filemmap == NULL) {
      CloseHandle(mapping);
      CloseHandle(filedes);
      mapping = 0;
      filedes = INVALID_HANDLE_VALUE;
      Warning("cannot mmap TFM name <%s> !\n", tfmname);
      return(false);
  }
  size = GetFileSize(filedes, NULL);
#else
  if ((filedes = open(tfmname,O_RDONLY)) == -1) {
    Warning("metric file %s could not be opened", tfmname);
    return(false);
  }
  fstat(filedes,&stat);
  filemmap = mmap(NULL,stat.st_size, PROT_READ, MAP_SHARED,filedes,0);
  if (filemmap == (unsigned char *)-1) {
    Warning("cannot mmap metric file %s",tfmname);
    close(filedes);
    return(false);
  }
#endif
  lh = UNumRead(filemmap+2,2);
  bc = UNumRead(filemmap+4,2);
  ec = UNumRead(filemmap+6,2);
  nw = UNumRead(filemmap+8,2);
  DEBUG_PRINT(DEBUG_TFM,(" %d %d %d %d",lh,bc,ec,nw));
  width=alloca(nw*sizeof(dviunits));  
  c=0;
  position=filemmap+24+(lh+ec-bc+1)*4;
  while( c < nw ) {
    width[c] = SNumRead(position,4);
    c++;
    position += 4; 
  }
  
  /* Read char widths */
  c=bc;
  position=filemmap+24+lh*4;
  while(c <= ec) {
    DEBUG_PRINT(DEBUG_TFM,("\n@%ld TFM METRICS:\t", 
		 (long)(position - filemmap)));
    tcharptr=xmalloc(sizeof(struct ft_char));
    tcharptr->data=NULL;
    tcharptr->tfmw=width[*position];
    DEBUG_PRINT(DEBUG_TFM,("%d [%d] %d",c,*position,tcharptr->tfmw));
    tcharptr->tfmw = (dviunits) 
      ((int64_t) tcharptr->tfmw * tfontp->s / (1 << 20));
    DEBUG_PRINT(DEBUG_TFM,(" (%d)",tcharptr->tfmw));
    if (c > NFNTCHARS) /* Only positive for now */
      Fatal("tfm character exceeds numbering limit");
    tfontp->chr[c] = tcharptr;
    c++;
    position += 4;
  }
#ifdef WIN32
  if (filemmap != (unsigned char *)-1 && UnmapViewOfFile(filemmap) == FALSE)
    Warning("cannot munmap TFM name file %s!?\n",tfmname);
  if (mapping != 0 && CloseHandle(mapping) == FALSE) 
      Warning("cannot close TFM name file mapping!?\n");
  if (filedes != INVALID_HANDLE_VALUE && CloseHandle(filedes) == FALSE)
    Warning("cannot close TFM name file %s!?\n", tfmname);
#endif
  return(true);
}
  

