################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/10/26 00:57:43 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1
USE_TEX = 1

!include <msvc/common.mak>

programs = $(objdir)\mf.exe $(objdir)\mf-nowin.exe 
!ifdef TEX_DLL
programs = $(programs) $(objdir)\$(library_prefix)mf.dll
libfiles = $(objdir)\$(library_prefix)mf.lib
!endif
manfiles = mf.1
#
# Object files
#
objects = $(objdir)\mf.obj $(objdir)\main.obj $(objdir)\trap.obj $(objdir)\epsf.obj $(objdir)\win32.obj

DEFS = -I.. $(DEFS) -DTEX_ENGINE -DHAVE_CONFIG_H # -DFOO -I../foo

#
# Main target
#
all: $(objdir) $(programs)

!ifdef TEX_DLL

$(objdir)\$(library_prefix)mf.exp: $(objdir)\$(library_prefix)mf.lib

$(objdir)\$(library_prefix)mf.lib: $(objects)
	$(archive) /DEF $(objects)

$(objdir)\$(library_prefix)mf.dll: $(objects) $(objdir)\$(library_prefix)mf.exp $(objdir)\mf.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link_dll) $(**) $(socketlibs) $(conlibs) gdi32.lib

$(objdir)\mf.exe: $(objdir)\main.obj $(objdir)\$(library_prefix)mf.lib $(texmfmplib)
	$(link) $(**) $(conlibs)
!else
$(objdir)\mf.exe: $(objects) $(objdir)\main.obj $(objdir)\mf.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(socketlibs) $(conlibs) gdi32.lib
!endif

$(objdir)\mf-nowin.exe: $(objdir)\mf.exe
	$(copy) $(objdir)\mf.exe $(objdir)\mf-nowin.exe

mf.c mfcoerce.h mfd.h: mf.p $(web2c_texmf) mfextra.h
	$(web2c) mf

#mf.p mf.pool: $(tangle) mf.web mf-w32.ch
#	$(tangle) mf.web mf-w32.ch
mf.p mf.pool: $(tangle) mf.web mf.ch
	$(tangle) mf.web mf.ch

mf-w32.ch: $(tie) mf.ch mf-supp-w32.ch
	$(tie) -c mf-w32.ch mf.web mf.ch mf-supp-w32.ch

mfextra.h: mfextra.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

mf.rc: mf.rc.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

# FIXME
testdir = ..\triptrap
dvitype_args = -output-level=2 -dpi=72.27 -page-start="*.*.*.*.*.*.*.*.*.*"

check:
	@echo <<.\mf-check.bat
PATH = .\$(objdir);$(kpathseadir)\$(objdir);$(geturldir)\$(objdir);c:\windows\system32;C:\WINDOWS\system32\wbem;c:\windows
	.\$(objdir)\mf --progname=mf "&./mf \tracingstats:=1; end."
	.\$(objdir)\mf --progname=mf "&./mf $(srcdir)\tests\online"
	.\$(objdir)\mf --progname=mf "&./mf $(srcdir)\tests\one.two"
	.\$(objdir)\mf --progname=mf "&./mf $(srcdir)\tests\uno.dos"
<<NOKEEP
	-@cmd /c .\mf-check.bat

# Can't run trap and mptrap in parallel, because both write trap.{log,tfm}.
trap: $(objdir)\mf.exe ..\texware\$(objdir)\tftopl.exe ..\mfware\$(objdir)\gftype.exe trap-clean
	@echo ">>> See $(testdir)\mftrap.diffs for example of acceptable diffs."
	@echo <<.\trap.bat
set TEXMFCNF=$(testdir)
rem Get same filename in log
@$(del) trap.mf & $(copy) $(testdir)\trap.mf . $(redir_stderr)
.\$(objdir)\mf --progname=inimf <$(testdir)\mftrap1.in > mftrapin.fot
$(move) trap.log mftrapin.log
$(diff) $(testdir)\mftrapin.log mftrapin.log
.\$(objdir)\mf --progname=inimf <$(testdir)\mftrap2.in >mftrap.fot
$(move) trap.log mftrap.log
$(move) trap.tfm mftrap.tfm
$(diff) $(testdir)\mftrap.fot mftrap.fot
$(diff) $(testdir)\mftrap.log mftrap.log
..\texware\$(objdir)\tftopl .\mftrap.tfm mftrap.pl
$(diff) $(testdir)\mftrap.pl mftrap.pl
..\mfware\$(objdir)\gftype -m -i .\trap.72270gf >trap.typ
$(diff) $(testdir)\trap.typ trap.typ
<<NOKEEP
	-@cmd /c .\trap.bat

trap-clean:
	-@echo $(verbose) & ( \
		for %%i in (trap.mf trap.base mftrapin.fot mftrapin.log \
                            mftrap.fot mftrap.log mftrap.tfm mftrap.pl  \
			    trap.72270gf trap.typ) do $(del) %%i $(redir_stderr) \
	)

!include <msvc/config.mak>
!include <msvc/install.mak>

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objdir)\mf.obj $(mf_c) mfextra.c mfcoerce.h mfd.h \
			    mf.p mf.pool mf.fmt mf.log mfextra.h mf.rc\
			    hello.dvi hello.log xfoo.out openout.log on.two.log uno.log \
			    just.log batch.log write18.log mlmftst.log mfput.log \
			    missfont.log) do $(del) %%i $(redir_stderr) \
	)
	-@$(deldir) tfm $(redir_stderr)

!include <msvc/clean.mak>
!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
