################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/09/17 12:20:18 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1
USE_TEX = 1


!include <msvc/common.mak>

programs =  $(objdir)\mpost.exe 
!ifdef TEX_DLL
programs = $(programs) $(objdir)\$(library_prefix)mpost.dll
libfiles = $(objdir)\$(library_prefix)mpost.lib
!endif
manfiles = mpost.1
#
# Object files
#
objects = $(objdir)\mp.obj $(objdir)\main.obj

DEFS = -I.. $(DEFS) -DTEX_ENGINE -DHAVE_CONFIG_H # -DFOO -I../foo

#
# Main target
#
all: $(objdir) $(programs)

!ifdef TEX_DLL

$(objdir)\$(library_prefix)mpost.exp: $(objdir)\$(library_prefix)mpost.lib

$(objdir)\$(library_prefix)mpost.lib: $(objects)
	$(archive) /DEF $(objects)

$(objdir)\$(library_prefix)mpost.dll: $(objects) $(objdir)\$(library_prefix)mpost.exp $(objdir)\mpost.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link_dll) $(**) $(socketlibs) $(conlibs)

$(objdir)\mpost.exe: $(objdir)\main.obj $(objdir)\$(library_prefix)mpost.lib $(texmfmplib)
	$(link) $(**) $(conlibs)
!else
$(objdir)\mpost.exe: $(objects) $(objdir)\main.obj $(objdir)\mpost.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(socketlibs) $(conlibs)
!endif

mp.c mpcoerce.h mpd.h: mp.p $(web2c_texmf) mpextra.h
	$(web2c) mp

#mp.p mp.pool: $(tangle) mp.web mp-w32.ch
#	$(tangle) mp.web mp-w32.ch
mp.p mp.pool: $(tangle) mp.web mp.ch
	$(tangle) mp.web mp.ch

#mp-w32.ch: $(tie) mp.ch mp-supp-w32.ch
#	$(tie) -c mp-w32.ch mp.web mp.ch mp-supp-w32.ch

mpextra.h: mpextra.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

mpost.rc: mpost.rc.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

# FIXME
testdir = ..\triptrap
dvitype_args = -output-level=2 -dpi=72.27 -page-start="*.*.*.*.*.*.*.*.*.*"

check:
	@echo <<.\mpost-check.bat
.\$(objdir)\mpost --progname=mpost "&./mpost \tracingstats:=1 ; end."
set MAKEMPX_BINDIR=.;..\xemtex\$(objdir)
set MPXCOMMAND=..\xemtex\$(objdir)\makempx.exe
.\$(objdir)\mpost --progname=mpost $(srcdir)\tests\mptest
set MAKEMPX_BINDIR=
set MPXCOMMAND=
.\$(objdir)\mpost --progname=mpost $(srcdir)\tests\one.two
.\$(objdir)\mpost --progname=mpost $(srcdir)\tests\uno.dos
<<NOKEEP
	-@cmd /c .\mpost-check.bat

# Can't run trap and mptrap in parallel, because both write trap.{log,tfm}.
mptrap: $(objdir)\mpost.exe ..\texware\$(objdir)\pltotf.exe ..\texware\$(objdir)\tftopl.exe mptrap-clean
	@echo ">>> See $(testdir)\mptrap.diffs for example of acceptable diffs." >&2
	@echo <<.\mptrap.bat
@echo $(verbose)
set TEXMFCNF=$(testdir)
rem Get same filename in log 
$(del) mtrap.mp & $(copy) $(testdir)\mtrap.mp . $(redir_stderr)
..\texware\$(objdir)\pltotf $(testdir)\trapf.pl trapf.tfm
.\$(objdir)\mpost -ini mtrap
$(diff) $(testdir)\mtrap.log mtrap.log
$(diff) $(testdir)\mtrap.0 mtrap.0
$(diff) $(testdir)\mtrap.1 mtrap.1
$(diff) $(testdir)\writeo writeo
$(diff) $(testdir)\writeo.2 writeo.2
@$(del) trap.mp & $(copy) $(testdir)\trap.mp . $(redir_stderr)
@$(del) trap.mpx & $(copy) $(testdir)\trap.mpx . $(redir_stderr)
.\$(objdir)\mpost -ini <$(testdir)\mptrap1.in >mptrapin.fot
$(move) trap.log mptrapin.log
$(diff) $(testdir)\mptrapin.log mptrapin.log
# Must run inimp or font_name[null_font] is not initialized, leading to diffs.
.\$(objdir)\mpost --ini <$(testdir)\mptrap2.in >mptrap.fot
$(move) trap.log mptrap.log
$(move) trap.tfm mptrap.tfm
$(diff) $(testdir)\mptrap.fot mptrap.fot
$(diff) $(testdir)\mptrap.log mptrap.log
$(diff) $(testdir)\trap.5 trap.5
$(diff) $(testdir)\trap.6 trap.6
$(diff) $(testdir)\trap.148 trap.148
$(diff) $(testdir)\trap.149 trap.149
$(diff) $(testdir)\trap.150 trap.150
$(diff) $(testdir)\trap.151 trap.151
$(diff) $(testdir)\trap.197 trap.197
$(diff) $(testdir)\trap.200 trap.200
..\texware\$(objdir)\tftopl .\mptrap.tfm mptrap.pl
$(diff) $(testdir)\mptrap.pl mptrap.pl
<<NOKEEP
	-@cmd /c .\mptrap.bat

mptrap-clean:
	-@echo $(verbose) & ( \
		for %%i in (mtrap.mp mtrap.mem trapf.tfm \
			    mtrap.log mtrap.0 mtrap.1 writeo writeo.log writeo.2 \
			    trap.mp trap.mpx mptrapin.fot mptrapin.log \
			    mptrap.fot mptrap.log mptrap.tfm \
			    trap.ps trap.mem trap.0 trap.5 trap.6 trap.95 trap.96 trap.97 \
			    trap.98 trap.99 trap.100 trap.101 trap.102 trap.103 trap.104 \
			    trap.105 trap.106 trap.107 trap.108 trap.109 trap.148 \
			    trap.149 trap.150 trap.151 trap.197 trap.200 \
			    mptrap.pl) do $(del) %%i $(redir_stderr) \
	)

!include <msvc/config.mak>
!include <msvc/install.mak>

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objdir)\mp.obj mp.c mpextra.c mpcoerce.h mpd.h \
			    mp.p mp.pool mp.fmt mp.log mpextra.h mpost.rc \
			    hello.dvi hello.log xfoo.out openout.log on.two.log uno.log \
			    just.log batch.log write18.log mpput.log \
			    missfont.log) do $(del) %%i $(redir_stderr) \
	)
	-@$(deldir) tfm $(redir_stderr)

!include <msvc/clean.mak>
!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
