################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/09/17 12:20:39 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1
USE_TEX = 1

!include <msvc/common.mak>

programs =  $(objdir)\omega.exe 
!ifdef TEX_DLL
programs = $(programs) $(objdir)\$(library_prefix)omega.dll
libfiles = $(objdir)\$(library_prefix)omega.lib
!endif
manfiles = omega.1
#
# Object files
#
# The C sources.
omega_c = omega.c
omega_o = $(objdir)\omega.obj

objects = $(objdir)\omega.obj $(objdir)\omegafirst.obj $(objdir)\omegabis.obj

DEFS = -I.. $(DEFS) -DTEX_ENGINE -DHAVE_CONFIG_H # -DFOO -I../foo

#
# Main target
#
all: $(objdir) $(programs)

!ifdef TEX_DLL

$(objdir)\$(library_prefix)omega.exp: $(objdir)\$(library_prefix)omega.lib

$(objdir)\$(library_prefix)omega.lib: $(objects)
	$(archive) /DEF $(objects)

$(objdir)\$(library_prefix)omega.dll: $(objects) $(objdir)\$(library_prefix)omega.exp $(objdir)\omega.res $(pdftexlibs) $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link_dll) $(**) $(socketlibs) $(conlibs)

$(objdir)\omega.exe: $(objdir)\main.obj $(objdir)\$(library_prefix)omega.lib $(texmfmplib)
	$(link) $(**) $(conlibs)
!else
$(objdir)\omega.exe: $(objects) $(objdir)\main.obj $(objdir)\omega.res $(pdftexlibs) $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(socketlibs) $(conlibs)
!endif

omega: $(omega)

# C file dependencies.
omega.c omegacoerce.h omegad.h: omega.p $(web2c_texmf)
	$(web2c) omega

# Tangling
omega.p omega.pool: $(otangle) omega.web omega.ch
	$(otangle) omega.web omega.ch

#
# e-Omega is build without the xml support
# so we can't reuse omega.web and omega.ch
# we need to build stripped down versions
#
omega_files = \
	omnode.ch \
	om16bit.ch \
	omstr.ch \
	omfont.ch \
	omchar.ch \
	omfi.ch \
	ompar.ch \
	omocp.ch \
	omfilter.ch \
	omtrans.ch \
	omdir.ch \
	omxml.ch

omega_changefiles = \
	comnode.ch \
	com16bit.ch \
	comstr.ch \
	comfont.ch \
	comchar.ch \
	comfi.ch \
	compar.ch \
	comocp.ch \
	comfilter.ch \
	comtrans.ch \
	comdir.ch \
	comxml.ch \
	comsrcspec.ch

# Generation of the web and ch files.
omega.web: $(tie) ..\tex\tex.web $(omega_files) \
	omegaextra.h omega.defines # omegadir\omega.mak
	$(tie) -m omega.web ..\tex\tex.web $(omega_files)

omega.ch: $(tie) omega.web $(omega_changefiles) # omegadir/omega.mak
	$(tie) -c omega.ch omega.web $(omega_changefiles)

omegaextra.h: omegaextra.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

omega.rc: omega.rc.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

test:
	echo $(PATH)
	fmtutil --byfmt=omega

# 

!include <msvc/config.mak>
!include <msvc/install.mak>

# Installation.
install::
	@$(copy) omega.pool "$(texpooldir)\omega.pool" $(redir_stdout)

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objects) omega.c omegaextra.c omegacoerce.h \
			    omegad.h omega.p omega.pool omega.web omega.ch \
			    omega.fmt omega.log omegaextra.h omega.rc) do \
				$(del) %%i $(redir_stderr) \
	)

!include <msvc/clean.mak>
!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
