################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/11/07 17:09:42 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1
USE_TEX = 1
USE_PDF = 1
USE_XPDF = 1
USE_JPEG = 1
USE_PNG = 1
USE_MD5 = 1
USE_ZLIB = 1

!include <msvc/common.mak>

programs = \
	$(objdir)\pdfetex.exe \
!ifndef XEMTEX
	$(objdir)\pdftex.exe \
!endif
!ifndef STANDALONE
!ifndef XEMTEX
	$(objdir)\ttf2afm.exe \
!endif
	$(objdir)\pdftosrc.exe
!endif

!ifdef TEX_DLL
programs = $(programs) $(objdir)\$(library_prefix)pdfetex.dll
libfiles = $(objdir)\$(library_prefix)pdfetex.lib
!endif
manfiles = pdfetex.1
#
# Object files
#
# The C sources.
pdfetex_c = pdfetex.c
pdfetex_o = $(objdir)\pdfetex.obj

objects = $(objdir)\pdfetex.obj

DEFS = -I.. $(DEFS) -DTEX_ENGINE -DHAVE_CONFIG_H # -DFOO -I../foo

#
# Main target
#
all: $(objdir) $(programs)

!ifdef TEX_DLL

$(objdir)\$(library_prefix)pdfetex.exp: $(objdir)\$(library_prefix)pdfetex.lib

$(objdir)\$(library_prefix)pdfetex.lib: $(objects)
	$(archive) /DEF $(objects)

$(objdir)\$(library_prefix)pdfetex.dll: $(objects) $(objdir)\$(library_prefix)pdfetex.exp $(objdir)\pdfetex.res $(pdflib) $(kpathsealib) $(texmfmplib) $(gnuw32lib) $(xpdflib) $(jpeglib) $(pnglib) $(zliblib) $(md5lib)
	$(link_dll) $(**) $(socketlibs) $(conlibs)

$(objdir)\pdfetex.exe: $(objdir)\main.obj $(objdir)\$(library_prefix)pdfetex.lib $(texmfmplib)
	$(link) $(**) $(conlibs)
!else
$(objdir)\pdfetex.exe: $(objects) $(objdir)\main.obj $(objdir)\pdfetex.res $(pdflib) $(kpathsealib) $(texmfmplib) $(gnuw32lib) $(xpdflib) $(jpeglib) $(pnglib) $(zliblib) $(md5lib)
	$(link) $(**) $(socketlibs) $(conlibs)
!endif

pdfetex: $(pdfetex)

$(objdir)\pdftex.exe: $(objdir)\pdfetex.exe
	-@$(copy) $(**) $(@) $(redir_stderr)

# C file dependencies.
pdfetex.c pdfetexcoerce.h pdfetexd.h: pdfetex.p $(web2c_texmf) pdfetexextra.h
	$(web2c) pdfetex

# Tangling
pdfetex.p pdfetex.pool: $(tangle) pdfetex.web pdfetex.ch
	$(tangle) pdfetex.web pdfetex.ch

pdfetex_files = \
	 ..\etex\etex.ch \
	 ..\etex\etex.fix \
         ..\pdfetex\pdfetex.ch1 \
         ..\pdftex\pdftex.ch \
         ..\pdftex\hz.ch \
         ..\pdftex\misc.ch \
         ..\pdftex\vadjust.ch \
         ..\pdfetex\pdfetex.ch2

pdfetex_changefiles = \
	    ..\pdfetex\tex.ch0 \
	    ..\tex\tex.ch \
#	    ..\tex\tex-supp-w32.ch \
	    ..\etex\tex.ch1 \
	    ..\etex\tex.ech \
	    ..\pdfetex\tex.ch1 \
	    ..\pdftex\tex.pch

# Generation of the web and ch files.
pdfetex.web: $(tie) ..\tex\tex.web $(pdfetex_files) \
	pdfetex.h pdfetex.defines # pdfetexdir\pdfetex.mak
	$(tie) -m pdfetex.web ..\tex\tex.web $(pdfetex_files)

pdfetex.ch: $(tie) pdfetex.web $(pdfetex_changefiles) # pdfetexdir/pdfetex.mak
	$(tie) -c pdfetex.ch pdfetex.web $(pdfetex_changefiles)

pdfetex.h: ..\pdftex\pdftex.h
	-@$(del) $(@) $(redir_stderr)
	$(copy) $(**) $(@)

pdfetex.defines: ..\pdftex\pdftex.defines
	-@$(del) $(@) $(redir_stderr)
	$(copy) $(**) $(@)

pdfetexextra.h: pdfetexextra.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

pdfetex.rc: pdfetex.rc.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

# 
$(objdir)\ttf2afm.exe: $(objdir)\ttf2afm.obj $(texmfmplib) $(kpathsealib) $(gnuw32lib)
	$(link) $(**) $(conlibs)

$(objdir)\ttf2afm.obj: ttf2afm.c ..\pdftexdir\macnames.c
	$(compile) -I..\pdftexdir ttf2afm.c

$(objdir)\pdftosrc.exe: $(objdir)\pdftosrc.obj $(pdflib) $(kpathsealib) $(texmfmplib) $(gnuw32lib) $(xpdflib) $(jpeglib) $(pnglib) $(zliblib) $(md5lib)
	$(link) $(**) $(conlibs)

$(objdir)\pdftosrc.obj: pdftosrc.cc
	$(CC) -I$(xpdfdir)\xpdf $(CFLAGS) -c /Fo$@ /Tppdftosrc.cc
# 

!include <msvc/config.mak>
!include <msvc/install.mak>

# Installation.
install::
	@$(copy) pdfetex.pool "$(texpooldir)\pdfetex.pool" $(redir_stdout)

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objects) pdfetex.c pdfetexextra.c pdfetexcoerce.h \
			    pdfetexd.h pdfetex.p pdfetex.pool pdfetex.web pdfetex.ch \
			    pdfetex.fmt pdfetex.log pdfetexextra.h pdfetex.rc \
			    $(objdir)\ttf2afm.obj $(objdir)\macnames.obj \
			    $(objdir)\pdftosrc.obj) do $(del) %%i $(redir_stderr) \
	)

!include <msvc/clean.mak>
!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
