################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/09/17 12:21:43 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1
USE_TEX = 1
USE_PDF = 1
USE_JPEG = 1
USE_PNG = 1
USE_ZLIB = 1
USE_XPDF = 1
USE_MD5 = 1

!include <msvc/common.mak>

programs = \
	$(objdir)\pdftex.exe \
!ifndef STANDALONE
	$(objdir)\ttf2afm.exe \
	$(objdir)\pdftosrc.exe
!endif

!ifdef TEX_DLL
programs = $(programs) $(objdir)\$(library_prefix)pdftex.dll
libfiles = $(objdir)\$(library_prefix)pdftex.lib
!endif
manfiles = pdftex.1
#
# Object files
#
# The C sources.
pdftex_c = pdftex.c
pdftex_o = $(objdir)\pdftex.obj

objects = $(objdir)\pdftex.obj

DEFS = -I.. $(DEFS) -DTEX_ENGINE -DHAVE_CONFIG_H # -DFOO -I../foo

#
# Main target
#
all: $(objdir) $(programs)

!ifdef TEX_DLL

$(objdir)\$(library_prefix)pdftex.exp: $(objdir)\$(library_prefix)pdftex.lib

$(objdir)\$(library_prefix)pdftex.lib: $(objects)
	$(archive) /DEF $(objects)

$(objdir)\$(library_prefix)pdftex.dll: $(objects) $(objdir)\$(library_prefix)pdftex.exp $(objdir)\pdftex.res $(pdflib) $(kpathsealib) $(texmfmplib) $(gnuw32lib) $(xpdflib) $(jpeglib) $(pnglib) $(zliblib) $(md5lib)
	$(link_dll) $(**) $(socketlibs) $(conlibs)

$(objdir)\pdftex.exe: $(objdir)\main.obj $(objdir)\$(library_prefix)pdftex.lib $(texmfmplib)
	$(link) $(**) $(conlibs)
!else
$(objdir)\pdftex.exe: $(objects) $(objdir)\main.obj $(objdir)\pdftex.res $(pdflib) $(kpathsealib) $(texmfmplib) $(gnuw32lib) $(xpdflib) $(jpeglib) $(pnglib) $(zliblib) $(md5lib)
	$(link) $(**) $(socketlibs) $(conlibs)
!endif

pdftex: $(pdftex)

# C file dependencies.
pdftex.c pdftexcoerce.h pdftexd.h: pdftex.p $(web2c_texmf) pdfextra.h
	$(web2c) pdftex

# Tangling
pdftex.p pdftex.pool: $(tangle) pdftex.web xpdftex.ch
	$(tangle) pdftex.web xpdftex.ch

pdftex_files = \
         pdftex.ch	\
         hz.ch 		\
         misc.ch	\
	..\pdfxtex\vadjust.ch

pdftex_changefiles = \
	   tex.ch0			\
	   ..\tex\tex.ch		\
#	   ..\tex\tex-supp-w32.ch	\
	   tex.ch1			\
	   tex.pch

# Generation of the web and ch files.
pdftex.web: $(tie) ..\tex\tex.web $(pdftex_files) \
	pdftex.h pdftex.defines # pdftexdir\pdftex.mak
	$(tie) -m pdftex.web ..\tex\tex.web $(pdftex_files)

xpdftex.ch: $(tie) pdftex.web $(pdftex_changefiles) # pdftexdir/pdftex.mak
	$(tie) -c xpdftex.ch pdftex.web $(pdftex_changefiles)

pdftexextra.h: pdftexextra.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

pdftex.rc: pdftex.rc.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

# 
$(objdir)\ttf2afm.exe: $(objdir)\ttf2afm.obj $(texmfmplib) $(kpathsealib) $(gnuw32lib)
	$(link) $(**) $(conlibs)

$(objdir)\ttf2afm.obj: ttf2afm.c ..\pdftexdir\macnames.c
	$(compile) -I..\pdftexdir ttf2afm.c

$(objdir)\pdftosrc.exe: $(objdir)\pdftosrc.obj $(pdflib) $(kpathsealib) $(texmfmplib) $(gnuw32lib) $(xpdflib) $(jpeglib) $(pnglib) $(zliblib) $(md5lib)
	$(link) $(**) $(conlibs)

$(objdir)\pdftosrc.obj: pdftosrc.cc
	$(CC) -I$(xpdfdir)\xpdf $(CFLAGS) -c /Fo$@ /Tp$(**)
#
!include <msvc/config.mak>
!include <msvc/install.mak>

# Installation.
install::
	@$(copy) pdftex.pool "$(texpooldir)\pdftex.pool" $(redir_stdout)

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objects) pdftex.c pdftexextra.c pdftexcoerce.h \
			    pdftexd.h pdftex.p pdftex.pool pdftex.web xpdftex.ch \
			    pdftex.fmt pdftex.log pdftexextra.h pdftex.rc \
			    $(objdir)\ttf2afm.obj $(objdir)\macnames.obj \
			    $(objdir)\pdftosrc.obj) do $(del) %%i $(redir_stderr) \
	)

!include <msvc/clean.mak>
!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
