################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/09/17 12:22:45 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1
USE_TEX = 1

!include <msvc/common.mak>

programs =  $(objdir)\tex.exe 
!ifdef TEX_DLL
programs = $(programs) $(objdir)\$(library_prefix)tex.dll
libfiles = $(objdir)\$(library_prefix)tex.lib
!endif
manfiles = tex.1
#
# Object files
#
objects = $(objdir)\tex.obj $(objdir)\main.obj

DEFS = -I.. $(DEFS) -DTEX_ENGINE -DHAVE_CONFIG_H # -DFOO -I../foo

#
# Main target
#
all: $(objdir) $(programs)

!ifdef TEX_DLL

$(objdir)\$(library_prefix)tex.exp: $(objdir)\$(library_prefix)tex.lib

$(objdir)\$(library_prefix)tex.lib: $(objdir)\tex.obj
	$(archive) /DEF $(objdir)\tex.obj

$(objdir)\$(library_prefix)tex.dll: $(objdir)\tex.obj $(objdir)\$(library_prefix)tex.exp $(objdir)\tex.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link_dll) $(**) $(socketlibs) $(conlibs)

$(objdir)\tex.exe: $(objdir)\main.obj $(objdir)\$(library_prefix)tex.lib $(texmfmplib)
	$(link) $(**) $(conlibs)
!else
$(objdir)\tex.exe: $(objdir)\tex.obj $(objdir)\main.obj $(objdir)\tex.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(socketlibs) $(conlibs)
!endif

tex.c texcoerce.h texd.h: tex.p $(web2c_texmf) texextra.h
	$(web2c) tex

# tex.p tex.pool: $(tangle) tex.web tex-w32.ch
#	$(tangle) tex.web tex-w32.ch
tex.p tex.pool: $(tangle) tex.web tex.ch
	$(tangle) tex.web tex.ch

# tex-w32.ch: $(tie) tex.ch tex-supp-w32.ch
# 	$(tie) -c tex-w32.ch tex.web tex.ch tex-supp-w32.ch

texextra.h: texextra.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

tex.rc: tex.rc.in
	$(perl) $(win32perldir)\configure-version.pl $(**) $@

# FIXME
testdir = ..\triptrap
dvitype_args = -output-level=2 -dpi=72.27 -page-start="*.*.*.*.*.*.*.*.*.*"

check:
	@echo <<.\tex-check.bat
PATH = $(PATH)
rem Test truncation (but don't bother showing the warning msg).
.\$(objdir)\tex --progname=tex --output-comment=$(outcom) $(srcdir)\tests\hello > nul \
  & .\$(objdir)\dvitype hello.dvi | grep olaf@infovore.xs4all.nl > nul
rem \openout should show up in \write's.
.\$(objdir)\tex --progname=tex $(srcdir)\tests\openout & grep xfoo openout.log
rem one.two.tex -> one.two.log
.\$(objdir)\tex --progname=tex $(srcdir)\tests\one.two & dir /n one.two.log
rem uno.dos -> uno.log
.\$(objdir)\tex --progname=tex $(srcdir)\tests\uno.dos & dir /n uno.log
.\$(objdir)\tex --progname=tex $(srcdir)\tests\just.texi & dir /n just.log
.\$(objdir)\tex --progname=tex $(srcdir)\tests\batch.tex
.\$(objdir)\tex --progname=tex --shell $(srcdir)\tests\write18 | grep echo
.\$(objdir)\tex --mltex --progname=initex --ini $(srcdir)\tests\mltextst
.\$(objdir)\tex --progname=tex <nul
set WEB2C=$(kpathsea_srcdir)
set TMPDIR=..
.\$(objdir)\tex --progname=tex "\nonstopmode\font\foo=nonesuch\end"
<<NOKEEP
	-@cmd /c .\tex-check.bat

trip: ..\texware\$(objdir)\dvitype.exe ..\texware\$(objdir)\pltotf.exe ..\texware\$(objdir)\tftopl.exe $(objdir)\tex.exe trip-clean
	@echo ">>> See $(testdir)\trip.diffs for example of acceptable diffs."
	@echo <<.\trip.bat
@echo $(verbose)
PATH = $(PATH)
set TEXMFCNF=$(testdir)
PATH = .\$(objdir);$(kpathseadir)\$(objdir);$(geturldir)\$(objdir);$(gnuw32dir)\$(objdir);$(texmfmpdir)\$(objdir);c:\windows\system32;C:\WINDOWS\system32\wbem;c:\windows
..\texware\$(objdir)\pltotf $(testdir)\trip.pl trip.tfm
..\texware\$(objdir)\tftopl .\trip.tfm trip.pl
$(diff) $(testdir)\trip.pl trip.pl
rem get same filename in log 
$(del) trip.tex & $(copy) $(testdir)\trip.tex . $(redir_stderr)
.\$(objdir)\tex --ini --progname=initex < $(testdir)\trip1.in >tripin.fot
$(move) trip.log tripin.log
$(diff) $(testdir)\tripin.log tripin.log
rem May as well test non-ini second time through.
.\$(objdir)\tex --progname=tex < $(testdir)\trip2.in >trip.fot
$(diff) $(testdir)\trip.fot trip.fot
rem We use $(diff) instead of `diff' only for those files where there
rem might actually be legitimate numerical differences.
$(diff) $(diffflags) $(testdir)\trip.log trip.log
..\texware\$(objdir)\dvitype $(dvitype_args) trip.dvi >trip.typ
$(diff) $(diffflags) $(testdir)\trip.typ trip.typ
<<KEEP
	-@cmd /c .\trip.bat

trip-clean:
	-@echo $(verbose) & ( \
		for %%i in (trip.tfm trip.pl trip.tex trip.fmt tripin.fot tripin.log \
			    trip.fot trip.log trip.dvi trip.typ tripos.tex 8terminal.tex \
			    ) do $(del) %%i $(redir_stderr) \
	)
	-@$(deldir) tfm $(redir_stderr)

!include <msvc/config.mak>
!include <msvc/install.mak>

# Installation.
install::
	@$(copy) tex.pool "$(texpooldir)\tex.pool" $(redir_stdout)

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objdir)\tex.obj tex.c texextra.c texcoerce.h texd.h \
			    tex.p tex.pool tex.fmt tex.log texextra.h tex.rc \
			    hello.dvi hello.log xfoo.out openout.log on.two.log uno.log \
			    just.log batch.log write18.log mltextst.log texput.log \
			    missfont.log) do $(del) %%i $(redir_stderr) \
	)
	-@$(deldir) tfm $(redir_stderr)

!include <msvc/clean.mak>
!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
