/* cmds.h -- declarations for cmds.c.
   $Id: cmds.h,v 1.5 2004/04/07 20:17:38 karl Exp $

   Copyright (C) 1998, 1999, 2002, 2003, 2004 Free Software Foundation,
   Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifndef CMDS_H
#define CMDS_H

/* The three arguments a command can get are a flag saying whether it is
   before argument parsing (START) or after (END), the starting position
   of the arguments, and the ending position.  */
typedef void CDECL COMMAND_FUNCTION (); /* So we can say COMMAND_FUNCTION *foo; */

/* Each command has an associated function.  When the command is
   encountered in the text, the associated function is called with START
   as the argument.  If the function expects arguments in braces, it
   remembers itself on the stack.  When the corresponding close brace is
   encountered, the function is called with END as the argument. */
#define START 0
#define END 1

/* Does the command expect braces?  */
#define NO_BRACE_ARGS 0
#define BRACE_ARGS 1
#define MAYBE_BRACE_ARGS 2

typedef struct
{
  char *name;
  COMMAND_FUNCTION *proc;
  int argument_in_braces;
} COMMAND;

extern COMMAND command_table[];

typedef struct acronym_desc
{
  struct acronym_desc *next;
  char *acronym;
  char *description;
} ACRONYM_DESC;

/* Texinfo commands.  */
extern void CDECL insert_self (int arg),
  CDECL insert_space (int arg),
  CDECL cm_ignore_line (void),
  CDECL cm_ignore_arg (int arg, int start_pos, int end_pos),
  CDECL cm_comment (void),
  CDECL cm_no_op (void);

/* Document structure and meta information.  */
extern void CDECL cm_setfilename (void),
  CDECL cm_settitle (void),
  CDECL cm_documentdescription (void),
  CDECL cm_node (void),
  CDECL cm_menu (void),
  CDECL cm_detailmenu (void),
  CDECL cm_dircategory (void),
  CDECL cm_direntry (void),
  CDECL cm_bye (void);

/* File inclusion.  */
extern void CDECL cm_include (void),
  CDECL cm_verbatiminclude (void);

/* Cross referencing commands.  */
extern void CDECL cm_anchor (int arg),
  CDECL cm_xref (int arg),
  CDECL cm_pxref (int arg),
  CDECL cm_ref (int arg),
  CDECL cm_inforef (int arg),
  CDECL cm_uref (int arg);

/* Special insertions.  */
extern void CDECL cm_LaTeX (int arg),
  CDECL cm_TeX (int arg),
  CDECL cm_bullet (int arg),
  CDECL cm_colon (void),
  CDECL cm_comma (int arg),
  CDECL cm_copyright (int arg),
  CDECL cm_dots (int arg),
  CDECL cm_enddots (int arg),
  CDECL cm_equiv (int arg),
  CDECL cm_error (int arg),
  CDECL cm_expansion (int arg),
  CDECL cm_image (int arg),
  CDECL cm_insert_copying (void),
  CDECL cm_minus (int arg),
  CDECL cm_point (int arg),
  CDECL cm_print (int arg),
  CDECL cm_punct (int arg),
  CDECL cm_registeredsymbol (int arg),
  CDECL cm_result (int arg);

/* Emphasis and markup.  */
extern void CDECL cm_acronym (int arg),
  CDECL cm_b (int arg),
  CDECL cm_cite (int arg, int position),
  CDECL cm_code (int arg),
  CDECL cm_dfn (int arg, int position),
  CDECL cm_dmn (int arg),
  CDECL cm_email (int arg),
  CDECL cm_emph (int arg),
  CDECL cm_i (int arg),
  CDECL cm_kbd (int arg),
  CDECL cm_key (int arg),
  CDECL cm_math (int arg),
  CDECL cm_not_fixed_width (int arg, int start, int end),
  CDECL cm_r (int arg),
  CDECL cm_sc (int arg, int start_pos, int end_pos),
  CDECL cm_strong (int arg, int start_pos, int end_pos),
  CDECL cm_tt (int arg),
  CDECL cm_indicate_url (int arg, int start, int end),
  CDECL cm_var (int arg, int start_pos, int end_pos),
  CDECL cm_verb (int arg);

/* Block environments.  */
extern void CDECL cm_cartouche (void),
  CDECL cm_group (void),
  CDECL cm_display (void),
  CDECL cm_smalldisplay (void),
  CDECL cm_example (void),
  CDECL cm_smallexample (void),
  CDECL cm_smalllisp (void),
  CDECL cm_lisp (void),
  CDECL cm_format (void),
  CDECL cm_smallformat (void),
  CDECL cm_quotation (void),
  CDECL cm_copying (void),
  CDECL cm_flushleft (void),
  CDECL cm_flushright (void),
  CDECL cm_verbatim (void),
  CDECL cm_end (void);

/* Tables, lists, enumerations.  */
extern void CDECL cm_table (void),
  CDECL cm_ftable (void),
  CDECL cm_vtable (void),
  CDECL cm_itemize (void),
  CDECL cm_enumerate (void),
  CDECL cm_multitable (void),
  CDECL cm_headitem (void),
  CDECL cm_item (void),
  CDECL cm_itemx (void),
  CDECL cm_tab (void);

extern void CDECL cm_center (void),
  CDECL cm_exdent (void),
  CDECL cm_indent (void),
  CDECL cm_noindent (void),
  CDECL cm_noindent_cmd (void);

/* Line and page breaks.  */
extern void CDECL cm_asterisk (void),
  CDECL cm_sp (void),
  CDECL cm_page (void);

/* Non breaking words.  */
extern void CDECL cm_tie (int arg),
  CDECL cm_w (int arg);

/* Title page creation.  */
extern void CDECL cm_titlepage (void),
  CDECL cm_author (void),
  CDECL cm_titlepage_cmds (void),
  CDECL cm_titlefont (int arg),
  CDECL cm_today (int arg);

/* Floats.  */
extern void CDECL cm_float (void),
  CDECL cm_caption (int arg),
  CDECL cm_shortcaption (void),
  CDECL cm_listoffloats (void);

/* Indices.  */
extern void CDECL cm_kindex (void),
  CDECL cm_cindex (void),
  CDECL cm_findex (void),
  CDECL cm_pindex (void),
  CDECL cm_vindex (void),
  CDECL cm_tindex (void),
  CDECL cm_defindex (void),
  CDECL cm_defcodeindex (void),
  CDECL cm_synindex (void),
  CDECL cm_printindex (void);

/* Conditionals. */
extern void CDECL cm_set (void),
  CDECL cm_clear (void),
  CDECL cm_ifset (void),
  CDECL cm_ifclear (void),
  CDECL cm_ifeq (void),
  CDECL cm_value (int arg, int start_pos, int end_pos);

#endif /* !CMDS_H */
