/* config.h: All .c files include this first.

Copyright (C) 1995, 96 Karl Berry.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef WEB2C_CONFIG_H
#define WEB2C_CONFIG_H

#if defined(WIN32)

# ifdef TEX_DLL
#  ifdef MAKE_TEX_DLL
#   define TEXDLL __declspec( dllexport)
#  else
#   define TEXDLL __declspec( dllimport)
#  endif
# else
#  define TEXDLL
# endif

# ifdef TEXMFMP_DLL
#  ifdef MAKE_TEXMFMP_DLL
#   define TEXMFMPDLL __declspec( dllexport)
#  else
#   define TEXMFMPDLL __declspec( dllimport)
#  endif
# else
#  define TEXMFMPDLL
# endif

#else /* ! WIN32 */

# define TEXDLL
# define TEXMFMPDLL

#endif

/* The stuff from the path searching library.  */
#include <kpathsea/config.h>
#ifdef WIN32
#include <texmfmp/c-auto.h>
#else
#include <web2c/c-auto.h>
#endif

#include <kpathsea/c-vararg.h>

/* How to open a binary file.  */
#include <kpathsea/c-fopen.h>

/* The smallest signed type: use `signed char' if ANSI C, `short' if
   char is unsigned, otherwise `char'.  */
#ifndef SCHAR_TYPE
#if __STDC__
#define SCHAR_TYPE signed char
#else /* not __STDC */
#ifdef __CHAR_UNSIGNED__
#define SCHAR_TYPE short
#else
#define SCHAR_TYPE char
#endif
#endif /* not __STDC__ */
#endif /* not SCHAR_TYPE */
typedef SCHAR_TYPE schar;

/* The type `integer' must be a signed integer capable of holding at
   least the range of numbers (-2^31)..(2^31-1).  If your compiler goes
   to great lengths to make programs fail, you might have to change this
   definition.  If this changes, you may have to modify
   web2c/fixwrites.c, since it generates code to do integer output using
   "%ld", and casts all integral values to be printed to `long'.
   
   If you define your own INTEGER_TYPE, you have to define your own
   INTEGER_MAX and INTEGER_MIN, too. */
#ifndef INTEGER_TYPE

#if SIZEOF_LONG > 4 && !defined (NO_DUMP_SHARE)
/* If we have 64-bit longs and want to share format files (with 32-bit
   machines), use `int'.  */
#define INTEGER_IS_INT
#endif

#ifdef INTEGER_IS_INT
#define INTEGER_TYPE int
#define INTEGER_MAX INT_MAX
#define INTEGER_MIN INT_MIN
#else
#define INTEGER_TYPE long
#define INTEGER_MAX LONG_MAX
#define INTEGER_MIN LONG_MIN
#endif /* not INTEGER_IS_INT */

#endif /* not INTEGER_TYPE */

typedef INTEGER_TYPE integer;

/* I don't want to write a configure test for remove when all Unix
   machines have unlink.  But, for the sake of non-Unix machines that
   support ANSI C... */
#if !defined (unix) && !defined (__unix__) && defined (__STDC__) && !defined (unlink)
#define unlink remove
#endif

/* Window support on the Amiga is just for the Amiga.  */
#ifdef AMIGA
#define AMIGAWIN
#endif

/* Window support for WIN32 machines. */
#ifdef WIN32
#define WIN32WIN
#endif

/* strtol.c */
#ifndef HAVE_STRTOL
extern TEXMFMPDLL long strtol P3H(const char *, char **, int);
#endif

/* Declarations for the routines we provide ourselves in lib/.  */

extern TEXMFMPDLL string basenamechangesuffix P3H(const_string,const_string,const_string);
extern TEXMFMPDLL string chartostring P1H(char);
extern TEXMFMPDLL boolean eof P1H(FILE *);
extern TEXMFMPDLL boolean eoln P1H(FILE *);
extern TEXMFMPDLL void readln P1H(FILE *);
extern TEXMFMPDLL void fprintreal P4H(FILE *, double, int, int);
extern TEXMFMPDLL integer inputint P1H(FILE *);
extern TEXMFMPDLL void printversionandexit P3H(const_string, const_string, const_string);
extern TEXMFMPDLL void zinput2ints P2H(integer *, integer *);
extern TEXMFMPDLL void zinput3ints P3H(integer *, integer *, integer *);
extern TEXMFMPDLL integer zround P1H(double);

/* main.c */
extern TEXMFMPDLL int argc;
extern TEXMFMPDLL string *argv;
extern TEXMFMPDLL string cmdline P1H(int);
extern TEXDLL void mainbody P1H(void); /* generated by web2c */

/* openclose.c */
extern TEXMFMPDLL boolean open_input P3H(FILE **, int, const_string fopen_mode);
extern TEXMFMPDLL boolean open_output P2H(FILE **, const_string fopen_mode);
extern TEXMFMPDLL void aclose P1H(FILE *);
extern TEXMFMPDLL void recorder_change_filename P1H(string);
extern TEXMFMPDLL boolean recorder_enabled;

/* version.c */
extern TEXMFMPDLL string versionstring;

/* oem.c */
#ifdef OEM
extern TEXMFMPDLL boolean bOem;
extern TEXMFMPDLL int fprintfOem(FILE *f, const char *fmt, ...);
extern TEXMFMPDLL int FputsOem(FILE *f, char *s);
extern TEXMFMPDLL int putcOem(int c, FILE *f);
#endif

/* From uexit.c.  This is here because the lib/ and web2c/ routines
   themselves can use it, but they don't need cpascal.h.  */
extern TEXMFMPDLL void uexit P1H(int status);

/* usage.c */
extern TEXMFMPDLL void usage P1H(const_string progname);
extern TEXMFMPDLL void usagehelp P2H(const_string *message, const_string bug_email);

/* job.c */
extern TEXMFMPDLL void texmfstartjob();
extern TEXMFMPDLL void __cdecl texmffinishjob();

#endif /* not CONFIG_H */
