/*
  Those functions are called whenever a job is started / finished.
  Currently, they provide a timing facility for Windows users.
*/

#include "config.h"
#include <kpathsea/config.h>
#include <kpathsea/c-ctype.h>

#include <time.h> /* For `struct tm'.  */
#if defined (HAVE_SYS_TIME_H)
#include <sys/time.h>
#elif defined (HAVE_SYS_TIMEB_H)
#include <sys/timeb.h>
#endif

TEXMFMPDLL boolean timestatp = false;
TEXMFMPDLL clock_t clock_start;

TEXMFMPDLL void
texmfstartjob()
{
#ifdef TIME_STATS
  if (! timestatp)
    return;

  clock_start = clock();
#endif
}

TEXMFMPDLL void __cdecl
texmffinishjob()
{
#ifdef TIME_STATS
  clock_t clock_since_start;

  if (! timestatp)
    return;

  clock_since_start = (clock() - clock_start) * 1000 / CLOCKS_PER_SEC;

  fprintf (stderr, "gross execution time: %u ms\n", clock_since_start);

#ifdef WIN32
  if (GetVersion() < 0x80000000) {
    HINSTANCE hinstKernel;
    FARPROC pfGetProcessTimes;
    FILETIME ftCreate, ftExit, ftKernel, ftUser;
    LARGE_INTEGER tUser64;
    LARGE_INTEGER tKernel64;
    DWORD tUser, tKernel;
    hinstKernel = LoadLibrary("kernel32.dll");
    if (hinstKernel == 0) {
      return;
    }
    pfGetProcessTimes = GetProcAddress(hinstKernel, "GetProcessTimes");
    if (pfGetProcessTimes == 0) {
      return;
    }
    if (! GetProcessTimes(GetCurrentProcess(), &ftCreate, &ftExit,
			  &ftKernel, &ftUser)) {
      return;
    }
    tUser64.LowPart = ftUser.dwLowDateTime;
    tUser64.HighPart = ftUser.dwHighDateTime;
    tKernel64.LowPart = ftKernel.dwLowDateTime;
    tKernel64.HighPart = ftKernel.dwHighDateTime;
    tUser = (DWORD)(tUser64.QuadPart / 10000);
    tKernel = (DWORD)(tKernel64.QuadPart / 10000);
    fprintf (stderr, "user mode: %u ms, kernel mode: %u ms, total: %u",
	     tUser, tKernel, tUser + tKernel);
  }
#else /* not WIN32 */
#endif /* WIN32 */

#endif /* TIME_STATS */
}
