################################################################################
#
# Makefile  : 
# Author    : Fabrice Popineau <Fabrice.Popineau@supelec.fr>
# Platform  : Win32, Microsoft VC++ 6.0, depends upon fpTeX 0.5 sources
# Time-stamp: <04/09/21 09:19:09 popineau>
#
################################################################################
root_srcdir = ..\..
INCLUDE=$(INCLUDE);$(root_srcdir)

# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, see the file COPYING.

USE_GNUW32 = 1
USE_GETURL = 1
USE_KPATHSEA = 1
USE_TEXMFMP = 1

!include <msvc/common.mak>

!ifdef STATIC

programs = 

targets = \
	$(objdir)\tangle.exe \
	$(objdir)\weave.exe

!else

programs = \
	$(objdir)\tangle.exe \
	$(objdir)\weave.exe

targets =

!endif

scripts =
libfiles =
includefiles =
manfiles = tangle.1 weave.1
infofiles =
#
# Object files
#
objects = $(objdir)\tangle.obj $(objdir)\weave.obj

DEFS = -I.. $(DEFS)

#
# Main target
#
all: $(objdir) $(programs) $(targets)

#
# Link target. setargv.obj is provided in the compiler library directory.
#
$(objdir)\tangle.exe: $(objdir)\main.obj $(objdir)\tangle.obj $(objdir)\tangle.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(conlibs)
	$(make) tangleboot.p

$(objdir)\weave.exe: $(objdir)\main.obj $(objdir)\weave.obj $(objdir)\weave.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**)  $(conlibs)

tangle.c tangle.h: tangle.p
	$(web2c) tangle

tangle.p: $(objdir)\tangleboot.exe tangle.web tangle.ch
	@echo <<.\tangle.bat
@echo $(verbose)
set WEBINPUTS=$(srcdir)
.\$(objdir)\tangleboot tangle tangle
set WEBINPUTS=
<<NOKEEP
	.\tangle.bat

weave.c weave.h: $(web2c_programs) weave.p
	$(web2c) weave

weave.p: $(objdir)\tangle.exe weave.web weave.ch
	$(objdir)\tangle weave weave

$(objdir)\tangleboot.exe: $(objdir)\main.obj $(objdir)\tangleboot.obj $(objdir)\tangle.res $(kpathsealib) $(texmfmplib) $(gnuw32lib)
	$(link) $(**) $(conlibs)

tangleboot.c tangleboot.h: stamp-tangle $(web2c_programs)
	$(web2c) tangleboot

# tangleboot.p is in the distribution
stamp-tangle: tangleboot.p
	date /t >stamp-tangle & time /t >>stamp-tangle

# This is not run unless tangle.web or tangle.ch is changed.
# Only try to run ./tangle if it actually exists, otherwise
# just touch tangleboot.p and build tangle.
tangleboot.p: tangle.web tangle.ch
	@echo <<.\tangleboot.bat
echo off
set WEBINPUTS=$(srcdir)
if exist $(objdir)\tangle.exe (
  $(objdir)\tangle tangle tangle
  & $(move) tangle.p tangleboot.p
) else if exist tangleboot.p (
  $(gnubin)\touch tangleboot.p
) else (
  $(copy) $(srcdir)\tangleboot.p .
)
date /t >stamp-ctangle & time /t >>stamp-tangle
$(make) $(objdir)\tangle.exe
set WEBINPUTS=
<<NOKEEP
	.\tangleboot.bat

!include <msvc/config.mak>

!include <msvc/install.mak>

!ifdef XEMTEX

install::

!else

install:: install-exec # install-lib install-include

!endif

!include <msvc/clean.mak>

clean::
	-@echo $(verbose) & ( \
		for %%i in ($(objdir)\tangle.obj tangle.c tangle.h tangle.p   \
		            $(objdir)\tangleboot.exe $(objdir)\tangleboot.obj \
	                    tangleboot.c tangleboot.h                         \
                            $(objdir)\weave.obj weave.c weave.h weave.p) do $(del) %%i $(redir_stderr) \
	)

distclean::
	-@echo $(verbose) & ( \
		for %%i in (stamp-tangle) do $(del) %%i $(redir_stderr) \
	)

!include <msvc/rdepend.mak>
!include "./depend.mak"

# End of .mak
#
# Local Variables:
# mode: makefile
# End:
